exampleGenpres <- function() {
#' Example intro script for RPresence using expected-value data generated by \code{genpres}
#'
#'  Dummy function for help display
#'
#' @export
#' @seealso \code{\link{exampleGenpres}}, \code{\link{exampleBobcats}}, \code{\link{exampleSalamanders}}, \code{\link{exampleSkinks}}
#' @examples
#' #     First thing: clear the workspace, set working directory and
#' #     load the RPresence library...
#' rm(list=ls()); setwd('~'); library(RPresence);
#' #===========================================================
#' #     In many examples, input is simulated using program  GENPRES.
#' #     GENPRES creates an output file suitable for program PRESENCE
#' #     named GENPRES.PAO.  Once that file is created, we can read it
#' #     with the RPresence function, "readPao".  This function will
#' #     read the specially formatted file and create a PRESENCE data
#' #     object.
#' #
#' #     Alternatively, we can read detection history data from a CSV
#' #     file and create a PRESENCE data object using the RPresence
#' #     function, "createPao".  Site and survey covariates can be
#' #     read in and added to the PRESENCE data object.
#' #
#'
#' #    Specify location of GENPRES...
#' genpres='c:/program\ files/presence/genpres8.exe';
#' if (!file.exists(genpres)) genpres='c:/program\ files\ (x86)/presence/genpres8.exe'
#' if (!file.exists(genpres)) stop('GENPRES not found')
#' #'
#' # ---- Example 1: Test simple single-season model using Genpres ----
#' #                  to generate expected-value data...
#'
#' #   First, we'll specify input parameters to GENPRES in a character
#' #   string.  N=100 means 100 sites, T=4 means 4 surveys, psi=0.75
#' #   means initial occupancy is 75%, p=0.60 means detection probability
#' #   for each survey is 60%.
#' strng=paste(genpres,'N=100 T=4 psi=.75 p=0.60')
#'
#' #   Now, we can run GENPRES using the R function, "system"...
#' i=system(strng)
#'
#' #   The PRESENCE input file, "genpres.pao" should have been created now,
#' #   so we can read it and create a PRESENCE data object named "pdataobj"
#' #   using the RPresence function, "readPao".
#' pdataobj=readPao('genpres.pao')
#' #   If you're using R-Studio, you can examine the contents of this object
#' #   in the "Environment" tab of one of the other window panes.  If not,
#' #   you can view the structure of this object using the R command, "str".
#' print(str(pdataobj))
#' #
#' #   If you want to change any elements in the "pdataobj" object, you can
#' #   do it with R commands.  For example to rename the 4 surveys to
#' #   "Jan","Feb","Mar","Apr", we could run this command:
#' pdataobj$surveynames=c("Jan","Feb","Mar","Apr")
#' #   If we wanted to change the sitenames to "camera1","camera2",...
#' #   we could run the command:
#' pdataobj$unitnames=paste("camera",1:pdataobj$nunits,sep='')
#' #   Note:  "unit" is synonymous with "site" in RPresence.
#'
#' #   Now, we can run a model using the RPresence function, "occMod".  We
#' #   call the function with arguments:
#' #      data: PRESENCE data object created above, "pdataobj"
#' #      type: Pre-defined type of model; "so" means "Static occupancy"
#' #      model: list of formula for each parameter in the model...
#' #                psi~1 means occupancy is the same for all sites
#' #                p~1 means detection is the same for all sites and surveys.
#' #      outfile: Name to give output text file for the model. If this
#' #               argument is omitted, no output file is saved, but results
#' #               are still stored in the R variable, "m1".
#' m1=occMod(data=pdataobj, type='so', model=list(psi~1,p~1),outfile='psi()p().out')
#' #   Examine results in R-studio, or print on console...
#' print(str(m1))
#' #   Let's print the estimates of occupancy(psi) and detection(p) from this
#' #   model...
#' print(unique(m1$real$psi))
#' print(unique(m1$real$p))
#' #   Notice that the estimates match the input parameters we gave to GENPRES.
#' #   This is due to the fact that we generated "expected-value" data, as
#' #   opposed to "simulated" data.
#' #
#' #   If you want to see the full Presence output, open the file, "psi()p().out"
#' #   using Notepad.
#'
#' #   Next, we'll build a model with all 4 detection probabilities fixed to 1.0.
#' #   To do this, we create a data-frame with a variable named, "param".  This is a
#' #   vector of length 4, with names, "p1","p2","p3", and "p4".  Those names
#' #   correspond to the names of the detection parameters in the Presence output.
#' fixed=data.frame(param=paste0('p',1:4),value=rep(1,4))
#'
#' #   Now, run the model using the function, "occMod"...
#' m2=occMod(data=pdataobj, type='so', model=list(psi~1,p~1),
#'           fixed=fixed,outfile='psi()p1.out')
#'
#' #   Print estimates from model...
#' print(unique(m2$real$psi))
#' print(unique(m2$real$p))
#'
#' #   Model comparison: To create a model comparison table, call the function,
#' #   "createAicTable" with the argument containing a list variable containing
#' #   the 2 models...
#' tbl=createAicTable(list(m1,m2))
#' #   The createAicTable function creates a summary table of model names, AIC
#' #   values, number of parameters,...  Also included in this object are the
#' #   model results.  To print just the model comparison table, enter the
#' #   command:
#' print(tbl$table)
}
