#  genpres.R - genpres functions

genpresEV <- function(N=100,K=4,psi=.75,p=.4,fp=.0,b=.0,eps=.0,gam=.0,SPS=0) {
  #' generate expected value detection histories
  #'
  #' \code{genpresEV} generates expected value detection history frequencies for
  #' single or multi-season models (false-positives optional)
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy
  #'@param p probability of detection (not false positive)
  #'@param fp probability of false detection
  #'@param b probability of "sure" detection
  #'@param eps probability of local extinction between seasons
  #'@param gam probability of local colonization between seasons
  #'@param SPS number of surveys per season
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least 2^K detection-histories, so if K>8,
  #'        it may take a very long time to run.
  #'
  #'@seealso \code{\link{genpresEVms}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}}
  #'@examples
  #'\dontrun{
  #'    # generate expected value data for standard single-season model (no false positives)
  #' hstfrq=genpresEV(N=1000, K=4, psi=.8, p=.3)  # generate det. hists and freqs
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  #'    # generate expected value data for single-season model with false positive detections
  #' hstfrq=genpresEV(N=1000, K=4, psi=.8, p=.3, fp=0.1, b=0.5)  # generate det. hists and freqs
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  #'    # generate expected value data for multi-season model (no false positive detections)
  #' hstfrq=genpresEV(N=1000, K=4, psi=.8, p=.3, eps=.2, gam=.1, SPS=2)  # generate det. hists and freqs
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq,nsurveyseason=2)   #  create pao file
  #'}
  if (SPS==0) SPS=K
  if (length(p)==1) p=rep(p,K)
  if (length(fp)==1) fp=rep(fp,K)
  if (length(b)==1) b=rep(b,K)
  if (length(gam)==1) gam=rep(gam,K)
  if (length(eps)==1) eps=rep(eps,K)
  p=rbind(fp,p); b=rbind(0,b); phi=cbind(gam,1-eps)
  srv <- function(k,n,hst,isocc) {
    if (n>0) {
      seasn=floor(k/SPS)
      occ=n*phi[seasn,isocc+1]
      cap(k+1,occ,  hst,1) # generate det. histories for occupied sites
      cap(k+1,n-occ,hst,0) # generate det. histories for unoccupied sites
    }
  }
  cap <- function(j,n,hst,isocc) {  #  determine how many sites with detections or
    if (n>0) {                      #  non-detections (recursively) each survey
      ndet=n*p[isocc+1,j];
      nsure=ndet*b[isocc+1,j];
      unsure=ndet-nsure;
      notdet=n-ndet;
      hsure=hunsure=hnotdet=hst;
      hunsure[j]=1; hsure[j]=2
      if (j<K) {
        if ((j %% SPS)==0) {
          srv(j,nsure,hsure,isocc);
          srv(j,unsure,hunsure,isocc);
          srv(j,notdet,hnotdet,isocc)
        } else {
          cap(j+1,nsure,hsure,isocc);
          cap(j+1,unsure,hunsure,isocc);
          cap(j+1,notdet,hnotdet,isocc)
        }
      } else {
        s1=paste(hunsure,collapse='');
        s2=paste(hsure,collapse='');
        s0=paste(hnotdet,collapse='')
        p=parent.frame()
        e$xhst=c(e$xhst,s0,s1,s2);
        e$frq=c(e$frq,notdet,unsure,nsure)
      }
    }
  }
  occ=N*psi; notocc=N-occ; e=environment(); e$frq=e$xhst=NULL;
  cap(1,occ,   rep(0,K),1); # generate det. histories for occupied sites
  cap(1,notocc,rep(0,K),0)  # generate det. histories for unoccupied sites
  s=unique(e$xhst);
  sumfrq=sapply(1:length(s),function(i) sum(e$frq[e$xhst==s[i]])) # summarize freqs by det. hist.
  i=which(sumfrq>0);   #  omit det. hists with frq=0
  hst=matrix(as.numeric(unlist(strsplit(s[i],''))),ncol=K,byrow=TRUE);
  colnames(hst)=paste(1:K)
  return(list(hst=hst, frq=sumfrq[i]))
}

genpresSD <- function(N=100,K=4,psi=.75,p=.4,fp=.0,b=.0,eps=.0,gam=.0,SPS=0,cov=FALSE,seed=0) {
  #' generate simulated data (single or multi-season) detection histories
  #'
  #' \code{genpresSD} generates expected value detection history frequencies for
  #' single or multi-season models
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy OR a pair of beta values
  #'@param p probability of detection scaler or vector
  #'@param fp probability of false detection
  #'@param b probability of "sure" detection
  #'@param eps probability of local extinction between seasons
  #'@param gam probability of local colonization between seasons
  #'@param SPS number of surveys per season
  #'@param cov use parameters as beta values and generate covariates
  #'@param seed seed number for random number generation
  #'
  #'@details
  #'If cov is TRUE, the parameters psi,p,gam,eps should be vectors
  #'constaining 2 values each.  The first is an intercept, and the
  #'second is a covariate effect.  They will be used with randomly
  #'generated site covariates (survey covarite for p) to produce
  #'real parameter values for each site.
  #'The site and survey covariates will be returned in the return object.
  #'
  #'@export
  #'@return list with detection-history matrix(hst) and
  #'        vector of frequencies(frq)
  #'
  #'@seealso \code{\link{genpresSDms}},\code{\link{genpresEV}},
  #'          \code{\link{genpresEVms}}
  #'@examples
  #'\dontrun{
  #'# generate simulated value data for standard single-season model
  #' simdata=genpresSD(N=1000, K=4, psi=.8, p=.3)  # generate det. hists and freqs
  #'
  #' SDpao=createPao(simdata$hst, frq=simdata$frq)                  #  create pao file
  #'
  #'########################
  #'
  #'    # generate simulated value data for multi-season model
  #' simdata=genpresSD(N=1000, K=4, psi=.8, p=.3, eps=.2, gam=.1, SPS=2)  # generate det. hists and freqs
  #'
  #' SDpao=createPao(simdata$hst, frq=simdata$frq,nsurveyseason=2)   #  create pao file
  #'
  #'#########################
  #'
  #' #generate simulated value data for single-season model with covariates
  #'
  #'data=genpresSD(N=500, K=4,
  #'               cov=TRUE,     # set to true to generate covariates
  #'               psi=c(1, 2),  # beta values for psi : psi=exp(1+2*cov)/(1+exp(1+2*cov))
  #'               p=c(-.5, 1))  # beta values for p: p=exp(-.5+1*cov)/(1+exp(-.5+1*cov))
  #'#  note: unit covariate name will be "ucov" and values from std. normal distribution
  #'#        survey covarite name is "scov" with range similar to ucov
  #'
  #'#  create pao file
  #'pao=createPao(data$hst,          # detection-history data
  #'              unitcov=data$ucov,   #  unit(site) covariate data
  #'              survcov=as.vector(unlist(data$scov))) # survey covariate data)
  #'
  #'#  run standard single-season model with psi a function of unitcov
  #'#    and p a function of survey covariate
  #'m1=occMod(model=list(psi~ucov, p~survcov),data=pao)
  #'
  #'#  create sequence of site covariate values from -2 to 2
  #'newcov = data.frame(ucov=seq(-2,2,.1))
  #'#  get predicted psi values for those covariate values
  #'psi = predict(m1, newdata=newcov, param='psi')
  #'
  #'#  plot psi versus range of site covariate values
  #'#  with 95% confidence intervals
  #'plot(x=newcov$ucov, y=psi$est, type='l', main='psi with 95% conf. intervals')
  #'lines(x=newcov$ucov, y=psi$lower_0.95, lty=2)
  #'lines(x=newcov$ucov, y=psi$upper_0.95, lty=2)
  #'}

  if (seed>0) set.seed(seed)
  if (SPS==0) SPS=K
  M=K/SPS     # M=number of seasons
  ucov=scov=NULL
  if (cov) {
    ucov=runif(N)
    if (length(psi)!=2 || length(p)!=2) stop('psi and p must contain 2 beta values each.')
    psi=plogis(psi[1]+psi[2]*ucov)
    scov=matrix(runif(N*K),ncol=K)
    p=plogis(p[1]+p[2]*scov)
    if (sum(fp) != 0) {
      ucov=cbind(ucov,runif(N))
      fp=plogis(fp[1]+fp[2]*ucov[,2])
    }
    if (sum(b) != 0) {
      ucov=cbind(ucov,runif(N))
      b=plogis(b[1]+b[2]*ucov[,ncol(ucov)])
    }
    if (sum(eps) != 0) {
      ucov=cbind(ucov,runif(N))
      eps=plogis(eps[1]+eps[2]*ucov[,ncol(ucov)])
      eps=matrix(rep(eps,M),ncol=M)
    }
    if (sum(gam) != 0) {
      ucov=cbind(ucov,runif(N))
      gam=plogis(gam[1]+gam[2]*ucov[,ncol(ucov)])
      gam=matrix(rep(gam,M),ncol=M)
    }
  }

  if (length(psi)==1) psi=rep(psi,N)
  if (length(gam)==1 && !is.matrix(gam)) gam=matrix(gam,N,M)
  if (length(eps)==1 && !is.matrix(eps)) eps=matrix(eps,N,M)
  if (length(p)==1) p=matrix(p,N,K)
  if (length(fp)==1) fp=matrix(fp,N,K)
  if (length(b)==1) b=matrix(b,N,K)
  occ=runif(N)<psi
  det1=occ * (runif(N)<p[,1])        # unverified detections
  det2=det1 * (runif(N)<b[,1])       # verified detections
  det.fp=(1-occ) * (runif(N)<fp[,1]) # false-positive detections
  det=det1+det2+det.fp; seasn=0
  for (j in 2:K) {
    if ((j %% SPS)==1) {
      seasn=seasn+1
      gameps=gam[,seasn]
      gameps[occ==1]=1-eps[occ==1,seasn]
      occ=runif(N)<gameps
    }
    det1=occ * (runif(N)<p[,j])
    det2=det1 * (runif(N)<b[,j])
    det.fp=(1-occ) * (runif(N)<fp[,j])
    det=cbind(det, det1+det2+det.fp)
  }
  return(list(hst=det, frq=rep(1,nrow(det)),
              ucov=data.frame(ucov),
              scov=data.frame(cov=scov)))
}

genpresSDmm <- function(N=100, K=4, psi=.75, theta=.4, p=.4, methods=2,
                        gam=NA, eps=NA, SPS=8888,seed=0) {
  #' generate simulated single-season, multi-method data detection histories
  #'
  #' \code{genpresSDmm} generates stochastic detection history frequencies for
  #' single-season, multi-method models
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy OR a pair of beta values
  #'@param theta probability of local occupancy  or vector
  #'@param p probability of detection scaler or vector
  #'@param methods number of methods per survey
  #'@param gam probability of local colonization
  #'@param eps probability of local extinction
  #'@param SPS number of surveys per season
  #'@param seed seed number for random number generation
  #'
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'
  #'@seealso \code{\link{genpresSDms}},\code{\link{genpresEV}},
  #'          \code{\link{genpresEVms}}
  #'
  #'@examples
  #'\dontrun{
  #'    # generate simulated value data for standard single-season model
  #'    # with 500 sites, 2 methods for each of 4 surveys, psi=0.8,
  #'    # local occupancy = .2, .3, .4, .5 for surveys 1,2,3 and 4,
  #'    # detection probs = .3 for 1st method, .8 for 2nd method
  #'
  #' simdata=genpresSDmm(N=500,                   # 500 sites
  #'             K=8,                             # 4 surveys * 2 methods
  #'             psi=0.8,                         # occupancy prob.
  #'             theta=c(.2,.3,.4,.5),            # local occ. prob each survey
  #'             p=c(.3,.8, .3,.8, .3,.8, .3,.8), # det.prob each survey/method
  #'             methods=2)                         # 2 methods per survey
  #'
  #' SDpao=createPao(simdata$hst, frq=simdata$frq, methods=2)    #  create pao file
  #'
  #'#  run single-season, multi-method (so.mm) model
  #'m1=occMod(model=list(psi~1, p~METHOD, theta~SURVEY),data=pao, type="so.mm")
  #'}
  if (seed>0) set.seed(seed)
  theta=rep(theta,K)  # repeat theta's in case only 1 specified
  p=rep(p,K)          # repeat p's in case only 1 or 2 specified
  gam=rep(gam,K)
  eps=rep(eps,K)
  gam_eps=cbind(gam,eps)

  occ=runif(N)<psi;
  survey=0; det=NULL
  for (j in 1:K) {
    if ((j %% SPS)==0) { season=season+1; occ=runif(N)<gam_eps[season,occ+1]}
    if ((j %% methods)==1) { survey=survey+1; occ=runif(N)<theta[survey]}
    det=cbind(det, occ * (runif(N)<p[j]))
  }
  tbl=table(apply(det,1,paste,collapse=''))

  return(list(hst=det, frq=rep(1,nrow(det))))
}

genpresEVcd <- function(N=100, K=3, psi=.75, p=.4, theta=c(1,1), th0pi=0,
                        gamma=0, epsilon=0, SPS=NA) {
  #' generate expected value detection histories for corr. det. model
  #'
  #' \code{genpresEVcd} generates expected value detection history frequencies for
  #' single or multi-season correlated detection models
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy
  #'@param p probability of detection (not false positive)
  #'@param theta probabilities of local occupancy (th0, th1)
  #'@param th0pi probability of local occupancy before 1st survey
  #'@param gamma probability of colonization
  #'@param epsilon probability of extinction
  #'@param SPS number of surveys per season
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least 2^K detection-histories, so if K>8, it may take a
  #'       very long time to run.
  #'
  #'@seealso \code{\link{genpresEVms}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}}, \code{\link{genpresEV}}
  #'@examples
  #'\dontrun{
  #'    # generate expected value data for standard single-season model (no corr.det.)
  #' hstfrq=genpresEVcd(N=1000, K=4, psi=.8, p=0.3, theta=c(1,1), th0pi=0)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  #'    # generate expected value data for single-season model with corr.det.
  #' hstfrq=genpresEVcd(N=1000, K=4, psi=.8, p=.3, theta=c(.2, .7), th0pi=.4)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  #' rm(list=ls()); N=100; K=2; psi=.75; p=.4; theta=c(.3, .6); th0pi=0; gamma=0; epsilon=0; SPS=K
  #'}
  UNOCC=1; OCC=2
  if (is.na(SPS)) SPS=K
  gam_eps=c(gamma, 1-epsilon); last_survey_in_season=seq(SPS,K,SPS)

  srv <- function(j,n,hst,state) {
    nocc=n*gam_eps[state]; nunocc=n-nocc
    nlococc=nocc*th0pi; nlocunocc=nocc-nlococc
    cap(j+1 , nlococc   , hst , OCC   , OCC)
    cap(j+1 , nlocunocc , hst , UNOCC , OCC)
    cap(j+1 , nunocc    , hst , UNOCC , UNOCC)
  }
  cap <- function(j,n,hst,localstate, state) {
    if (n>0) {
      nlococc=n*theta[localstate]*(state-1); hst1=paste0(hst,'1')
      nlocunocc=(n-nlococc)*(state-1)      ; hst0=paste0(hst,'0')
      ndet=nlococc*p; notdet=nlococc-ndet
      nunocc= n-nlococc-nlocunocc
      if (j<K) {
        if (j %in% last_survey_in_season) {
          srv(j,nunocc   , hst0, UNOCC)
          srv(j,nlocunocc, hst0, OCC)
          srv(j,ndet     , hst1, OCC)
          srv(j,notdet   , hst0, OCC)
        }
        else {
          cap(j+1,nunocc   , hst0 , UNOCC , UNOCC)
          cap(j+1,nlocunocc, hst0 , UNOCC , OCC)
          cap(j+1,ndet     , hst1 , OCC   , OCC)
          cap(j+1,notdet   , hst0 , OCC   , OCC)
        }
      }
      else {
        if (is.na(e$frq[hst1])) e$frq[hst1]=ndet  else e$frq[hst1]=e$frq[hst1]+ndet
        if (is.na(e$frq[hst0]))
          e$frq[hst0]=notdet+nlocunocc+nunocc
        else
          e$frq[hst0]=e$frq[hst0]+notdet+nlocunocc+nunocc
      }
    }
    return()
  }
  nocc=N*psi; nunocc=N-nocc; e=environment();
  nlococc=nocc*th0pi; nlocunocc=nocc-nlococc
  e$frq=NULL; e$frq['asdf']=0
  cap(1,nunocc   , '', UNOCC, UNOCC)  # generate det. histories for unoccupied sites
  cap(1,nlococc  , '', OCC  , OCC);    # generate det. histories for locally occupied sites
  cap(1,nlocunocc, '', UNOCC, OCC) # generate det. histories for locally unoccupied sites
  e$frq=e$frq[names(e$frq)!='asdf'];
  hst=matrix(as.numeric(unlist(strsplit(names(e$frq),''))),ncol=K,byrow=T)
  names(e$frq)=NULL
  return(list(hst=hst, frq=e$frq))
}

genpresSDcd <- function(N=100, K=3, psi=.75, p=.4, theta=c(1,1), th0pi=0,
                        gamma=0, epsilon=0, SPS=NA, seed=0) {
  #' generate stochastic data detection histories for corr. det. model
  #'
  #' \code{genpresEVcd} generates stochastic data  detection history frequencies for
  #' single or multi-season correlated detection models
  #'@param N number of sites
  #'@param K number of surveys (segments for spatial replication)
  #'@param psi probability of occupancy
  #'@param p probability of detection (not false positive)
  #'@param theta probabilities of local occupancy (th0, th1)
  #'@param th0pi probability of local occupancy before 1st survey
  #'@param gamma probability of colonization
  #'@param epsilon probability of extinction
  #'@param SPS number of surveys per season
  #'@param seed seed number for random number generation
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least 2^K detection-histories, so if K>8, it may take a
  #'       very long time to run.
  #'
  #'@seealso \code{\link{genpresEVms}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}}, \code{\link{genpresEV}}
  #'@examples
  #'\dontrun{
  #'# generate expected value data for standard single-season model (no corr.det.)
  #' hstfrq=genpresEVcd(N=1000, K=4, psi=.8, p=0.3, theta=c(1,1), th0pi=0)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  #'    # generate expected value data for single-season model with corr.det.
  #' hstfrq=genpresEVcd(N=1000, K=4, psi=.8, p=.3, theta=c(.2, .7), th0pi=.4)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  #'rm(list=ls()); N=10000; K=4; psi=.75; p=.4; theta=c(.3, .6); th0pi=0; gamma=0; epsilon=0; SPS=K
  #'}
  if (seed>0) set.seed(seed)
  if (is.na(SPS)) SPS=K  # if SPS not defined, set equal to 1 (1 season)
  nseasons = K/SPS       # number of seasons

  gam_eps=rep(psi,N);  det = NULL # for occupancy compare rand# w/ psi initially

  for (seasn in (1:nseasons)) {
    occ = 0+(runif(N)<gam_eps)    #  compare rand# with gamma if unocc, eps if occ
    pi=rep(th0pi,N) ; loc_occ = occ*(runif(N)<pi)  # simulate local occ before 1st survey

    for (segment in 1:SPS) {
      pi = rep(theta[1],N); pi[loc_occ>0] = theta[2] # simulate local occupancy
      loc_occ = occ*(runif(N)<pi)     # compare rand# w/ th0 if prev local unocc
      det = cbind(det, loc_occ*(runif(N)<p))  # simulate detection
    }
    gam_eps=rep(gamma,N); gam_eps[occ>0] = 1-epsilon # set gam_eps for next season
  }

  frq = table(apply(det,1,paste,collapse='')) # summarize history frequencies
  hst = matrix(as.numeric(unlist(strsplit(names(frq),''))),ncol=K,byrow=T)

  names(frq)=NULL
  return(list(hst=hst, frq=frq))
}

genpresEVmm <- function(N=100, K=4, psi=.75, p=c(.4,.6), theta=0.5,
                        gamma=0, epsilon=0, methods=2, SPS=NA) {
  #' generate expected value detection histories for multi-method model
  #'
  #' \code{genpresEVcd} generates expected value detection history frequencies for
  #' single or multi-season correlated detection models
  #'@param N number of sites
  #'@param K total number of detection events = number of seasons * methods * SPS
  #'@param psi probability of occupancy
  #'@param p probability of detection for each method
  #'@param theta probability of local occupancy
  #'@param gamma probability of colonization
  #'@param epsilon probability of extinction
  #'@param methods number of methods per survey
  #'@param SPS number of surveys per season
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least 2^K detection-histories, so if K>8, it may take a
  #'       very long time to run.
  #'
  #'@seealso \code{\link{genpresEVms}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}}, \code{\link{genpresEV}}
  #'@examples
  #'\dontrun{
  #'    # generate expected value data for standard single-season model (no corr.det.)
  #' hstfrq=genpresEVmm(N=1000, K=4, psi=.8, p=c(.3,.3), theta=1)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  #'    # generate expected value data for single-season multi-method model
  #' hstfrq=genpresEVmm(N=1000, K=4, psi=.8, p=c(.3,.5), theta=.5, methods=2)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'}
  UNOCC=1; OCC=2
  if (is.na(SPS)) SPS=K/methods
  gam_eps=c(gamma, 1-epsilon);
  last_method_in_survey=seq(methods,K,methods);
  last_survey_in_season=seq(methods*SPS,K,methods*SPS)
  pp=rep(p,K); theta=rep(theta,K)

  srv <- function(j,n,hst,state) {
    nocc=n*(state-1)
    if (j %in% last_survey_in_season) nocc=n*gam_eps[state];
    if (j == 0) nocc=n*psi
    nunocc=n-nocc; nlococc=nocc*theta[max(1,j)]; nlocunocc=nocc-nlococc
    cap(j+1 , nlococc   , hst , OCC   , OCC)
    cap(j+1 , nlocunocc , hst , UNOCC , OCC)
    cap(j+1 , nunocc    , hst , UNOCC , UNOCC)
  }
  cap <- function(j,n,hst,localstate, state) {
    if (n>0) {
      nlococc=n*(localstate-1)*(state-1); hst1=paste0(hst,'1')
      nlocunocc=n*(state-1)-nlococc     ; hst0=paste0(hst,'0')
      ndet=nlococc*pp[j]; notdet=nlococc-ndet
      nunocc= n-nlococc-nlocunocc
      if (j<K) {
        if (j %in% last_method_in_survey) {
          srv(j,nunocc   , hst0, UNOCC)
          srv(j,nlocunocc, hst0, OCC)
          srv(j,ndet     , hst1, OCC)
          srv(j,notdet   , hst0, OCC)
        }
        else {
          cap(j+1,nunocc   , hst0 , UNOCC , UNOCC)
          cap(j+1,nlocunocc, hst0 , UNOCC , OCC)
          cap(j+1,ndet     , hst1 , OCC   , OCC)
          cap(j+1,notdet   , hst0 , OCC   , OCC)
        }
      }
      else {
        if (is.na(e$frq[hst1])) e$frq[hst1]=ndet  else e$frq[hst1]=e$frq[hst1]+ndet
        if (is.na(e$frq[hst0]))
          e$frq[hst0]=notdet+nlocunocc+nunocc
        else
          e$frq[hst0]=e$frq[hst0]+notdet+nlocunocc+nunocc
      }
    }
    return()
  }
  e=environment(); e$frq=NULL; e$frq['asdf']=0
  srv(0, N, '', UNOCC)
  e$frq=e$frq[names(e$frq)!='asdf'];
  hst=matrix(as.numeric(unlist(strsplit(names(e$frq),''))),ncol=K,byrow=T)
  names(e$frq)=NULL
  return(list(hst=hst, frq=e$frq))
}

genpresEVms <- function(N=100,K=4,psi=.5,R=.8,p1=.4,p2=.3,dlta=.6,nstates=3,
                        psi1=0,Cpsi=0,CR=0,phi=0,p=0,SPS=0) {
  #' generate expected value detection histories for multi-state models
  #'
  #' \code{genpresEVms} generates expected value detection history frequencies for
  #' single or multi-season multi-state models.
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy
  #'@param R probability of breeding (state=2), given occupancy
  #'@param p1 probability of detection, given occupied (state=1)
  #'@param p2 probability of detection, given occupied and breeding (state=2)
  #'@param dlta probability of detecting breeding, given occupancy state=2
  #'@param nstates number of states (including "not occupied state")
  #'@param psi1 probability vector of initial states (not occ, occ, occ_breeding)
  #'@param Cpsi conditional probability of occupancy, given previous state
  #'@param CR conditional probability of breeding, given occupancy and previous state
  #'@param phi transition probability matrix (an alternative to specifying psi,R,Cpsi,CR)
  #'@param p detection probability matrix (an alternative to specifying p1,p2,dlta)
  #'@param SPS number of surveys per season
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least nstates^(K-1) detection-histories, so if K>6, it may take a
  #'       very long time to run.
  #'@seealso \code{\link{genpresEV}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}}
  #'
  #'@examples
  #'\dontrun{
  #'    # generate expected value data for standard single-season model (psi,R parameterization)
  #' x1=genpresEVms(N=100,K=4,psi=.75,R=.8,p1=.4,p2=.3,dlta=.6,nstates=3,
  #'                        psi1=0,Cpsi=0,CR=0,phi=0,p=0,SPS=4)
  #'
  #'    # generate expected value data for multi-season model (psi,R parameterization)
  #' x2=genpresEVms(N=100,K=4,psi=.75,R=.8,p1=.4,p2=.3,dlta=.6,nstates=3,psi1=0,
  #'                Cpsi=c(.66,.1,.15),CR=c(.72,.77,0),phi=0,p=0,SPS=2)
  #'
  #'    # generate expected value data for multi-season model (phi parameterization)
  #' nstates=3; p=matrix(0,nstates,nstates);
  #' p[2,2]=.4; p[3,2]=.3*.4; p[3,3]=.3*.6
  #' psi1=c(.25,.75*(1-.8),.75*.8)
  #' phi=rbind(c(.66*(1-.72),.66*.72),c(.1*(1-.77),.1*.77),c(.15,.0))
  #' phi=cbind(1-rowSums(phi),phi)
  #' x3=genpresEVms(N=100,K=4,nstates=3,psi1=psi1,phi=phi,p=p,SPS=2)
  #'
  #'    # generate expected value data for multi-season model (phi parameterization)
  #' nstates=3; p=matrix(0,nstates,nstates)
  #' p[2,2]=p[3,2]=p[3,3]=.25
  #' psi1=c(.167,.5,.333)
  #' phi=rbind(c(.5,.333),c(.5,.333),c(.5,.333))
  #' phi=cbind(1-rowSums(phi),phi)
  #' x4=genpresEVms(N=100,K=4,nstates=3,psi1=psi1,phi=phi,p=p,SPS=2)
  #'
  #'    # generate expected value data for multi-season model (phi parameterization)
  #' nstates=4; p=matrix(0,nstates,nstates)
  #' p[2:4,2]=p[3:4,3]=p[4,4]=.2
  #' psi1=c(0,.4,.3,.2); psi1[1]=1-sum(psi1)
  #' phi=matrix(0,nstates,nstates)
  #' phi[,2]=.4; phi[,3]=.3; phi[,4]=.2; phi[,1]=1-rowSums(phi)
  #' x4=genpresEVms(N=100,K=4,nstates=4,psi1=psi1,phi=phi,p=p,SPS=2)
  #' EVpao=createPao(x4$hst,frq=x4$frq,nsurveyseason=2)   #  create pao file
  #'}
  if (SPS==0) SPS=K
  if (length(psi1)==1) {
    psi1=c(1-psi, psi*(1-R), psi*R)
    p=matrix(c(1,0,0, 1-p1,p1,0, 1-p2,p2*(1-dlta),p2*dlta),3,3,byrow=T)
    phi=cbind(Cpsi*(1-CR),Cpsi*CR); phi=cbind(1-rowSums(phi),phi)
  }
  seasn_end=seq(SPS,K,SPS);  p[,1]=1-rowSums(p[,-1]); p[1,1]=1

  srv <- function(j,n,hst,istate) {
    if (n>0) for (i in 1:nstates) cap(j+1,n*phi[istate,i],hst,i)
  }
  cap <- function(j,n,hst,istate) {  #  determine how many sites with detections or
    if (n>0) {                      #  non-detections (recursively) each survey
      ndet=n*p[istate,]; h=matrix(rep(hst,nstates),nstates,byrow=T); i=2:nstates; h[i,j]=i-1
      if (j<K) {
        for (i in 1:nstates)
          if (j %in% seasn_end) srv(j,ndet[i],h[i,],istate) else cap(j+1,ndet[i],h[i,],istate)
      } else {
        for (i in 1:nstates)
          if (ndet[i]>0) {
            e$xhst=c(e$xhst,paste(h[i,],collapse=''))
            e$frq=c(e$frq,ndet[i])
          }
      }
    }
  }
  occ=N*psi1; e=environment(); e$frq=e$xhst=NULL;
  for (i in 1:nstates) cap(1,occ[i],rep(0,K),i);   # generate det. histories for occupied sites
  s=unique(e$xhst);
  sumfrq=sapply(1:length(s),function(i) sum(e$frq[e$xhst==s[i]])) # summarize freqs by det. hist.
  i=which(sumfrq>0);   #  omit det. hists with frq=0
  hst=matrix(as.numeric(unlist(strsplit(s[i],''))),ncol=K,byrow=T);
  colnames(hst)=paste(1:K)
  return(list(hst=hst, frq=sumfrq[i]))
}

genpresSDms <- function(N=100,K=4,psi=.5,R=.8,p1=.4,p2=.3,dlta=.6,Cpsi=rep(.5,3),CR=rep(.8,3),SPS=0,seed=0) {
  #' generate SIMULATED detection histories for multi-state model w/ psi/R parameterization.
  #'
  #' \code{genpresSDms} generates simulated detection histories for
  #' single or multi-season multi-state models.
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy
  #'@param R probability of breeding (state=2), given occupancy
  #'@param p1 probability of detection, given occupied (state=1)
  #'@param p2 probability of detection, given occupied and breeding (state=2)
  #'@param dlta probability of detecting breeding, given occupancy state=2
  #'@param Cpsi conditional probability of occupancy, given previous state
  #'@param CR conditional probability of breeding, given occupancy and previous state
  #'@param SPS number of surveys per season
  #'@param seed seed number for random number generation
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@seealso \code{\link{genpresSD}},\code{\link{genpresEV}},
  #'          \code{\link{genpresEVms}}
  #'
  #'@examples
  #'\dontrun{
  #'    # generate expected value data for standard single-season model (psi,R parameterization)
  #' x1=genpresSDms(N=100,K=4,psi=.75,R=.8,p1=.4,p2=.3,dlta=.6,SPS=4)
  #'}
  #'
  #'N=500;K=4;psi=.75;R=.8;p1=.4;p2=.3;dlta=.6;Cpsi=c(.66,.1,.15);CR=c(.72,.77,0);SPS=2;seed=0
  if (seed>0) set.seed(seed)
  if (SPS==0) SPS=K

  newstate <- function(n, psi, R) {  #  simulate which state each site is in
                                     #  (1)unocc, (2) occ-non-brd, (3)breeding
    st=1+(runif(n)<psi);  i=which(st>1);  st[i]=st[i]+(runif(length(i))<R)
    return(st)
  }
  state=newstate(N, psi, R)  #  assign state (1,2,3) to each site

  dhist=NULL; px=c(0, p1, p2)
  for (i in 1:K) {
    det=0+(runif(N)<px[state])   # determine if each site detected
    j=which(state==3 & det>0)    # select sites in state 3 with detection
    det[j]=det[j]+(runif(length(j))<dlta) # add 1 if detect breeding
    dhist=cbind(dhist,det)
    if ((i %% SPS) == 0 && i < K) {   # get new state for each site, depending
      j1=which(state==1); j2=which(state==2); j3=which(state==3)# on current
      state[j1]=newstate(length(j1), Cpsi[1], CR[1])            #  state
      state[j2]=newstate(length(j2), Cpsi[2], CR[2])
      state[j3]=newstate(length(j3), Cpsi[3], CR[3])
    }
  }
  shist=apply(dhist,1,paste,collapse='')
  x=table(shist)
  dhist=matrix(as.numeric(unlist(strsplit(names(x),''))),ncol=K,byrow=T)
  return(list(hst=dhist, frq=as.numeric(x)))
}

genpresEV2sp <- function(N=100,K=4,
                         psi=c(.8,.3,.7),
                         p=c(.6,.66,.7,.4,.5),
                         gam=c(0,0,0),eps=c(0,0,0),
                         SPS=0, param=0) {
  #' generate expected value detection histories for multi-state models
  #'
  #' \code{genpresEV2sp} generates expected value detection history frequencies for
  #' single or multi-season 2-species models.
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy (psiA, psiBA, psiBa)
  #'@param p probability of detection, given occupied state
  #'@param gam prob. colonization (gamA, gamBA, gamBa)
  #'@param eps prob. extinction (epsA, epsBA, epsBa)
  #'@param SPS number of surveys per season
  #'@param param 0=default=psiA, psiBA, psiBa parm, 1=psiA,psiB,psiAB parm
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least 3^(K-1) detection-histories,
  #'          so if K>6, it may take a very long time to run.
  #'@seealso \code{\link{genpresEV}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}},  \code{\link{genpresEVms}}
  #'
  #'@examples
  #'\dontrun{
  #' # generate expected value data for 2-species single-season model
  #' x1=genpresEV2sp(N=100,K=4,
  #'             psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
  #'             p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
  #'             gam=c(0, 0, 0),             #  gamA, gamBA, gamBa
  #'             eps=c(0, 0, 0),             #  epsA, epsBA, epsBa
  #'             SPS=4)                      #  surveys per season
  #'
  #' # generate expected value data for 2-species multi-season model
  #' x2=genpresEV2sp(N=100,K=4,              # 100 sites, 4 surveys
  #'             psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
  #'             p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
  #'             gam=c(.1, .14, .17),        #  gamA, gamBA, gamBa
  #'             eps=c(.2, .22, .24),        #  epsA, epsBA, epsBa
  #'             SPS=2)                      #  surveys per season
  #'}
  if (SPS==0) SPS=K
  psiA=psi[1]; psiBA=psi[2]; psiBa=psi[3]
  if (param==1) {
    # psiAB = psiA * psiBA               -> psiBA = psiAB/psiA
    # psiB = psiA*psiBA + (1-psiA)*psiBa -> psiBa = (psiB-psiA*psiBA)/(1-psiA)
    psiBA <- psi[3]/psiA; psiBa <- (psi[2] - psi[3]) /(1-psiA)
  }
  psi1=c(psiA*(1-psiBA), (1-psiA)*psiBa, psiA*psiBA)
  psi1=c(psi1,1-sum(psi1))

  if (length(gam)==3) gam[6]=gam[5]=gam[3]; gam[4]=gam[3]=gam[2]; gam[2]=gam[1]
  if (length(eps)==3) eps[6]=eps[5]=eps[3]; eps[4]=eps[3]=eps[2]; eps[2]=eps[1]

  gAB=gam[1]; gAb=gam[2]; gBAA=gam[3]; gBAa=gam[4]; gBaA=gam[6]; gBaa=gam[6]
  eAB=eps[1]; eAb=eps[2]; eBAA=eps[3]; eBAa=eps[4]; eBaA=eps[6]; eBaa=eps[6]

  #                  A0               0B               AB
  phi=matrix( c( (1-eAb)*(1-gBAA),     eAb*gBAa    ,   (1-eAb)*gBAA  , # A0
                 gAB*eBaA        , (1-gAB)*(1-eBaa),   gAB*(1-eBaA)  , # 0B
                 (1-eAB)*eBAA    ,   eAB*(1-eBAa)  , (1-eAB)*(1-eBAA), # AB
                 gAb*(1-gBaa)   , (1-gAb)*gBaa    ,  gAb*gBaA      ), # 00
              nrow=4,byrow=T)
  phi=cbind(phi,1-rowSums(phi))

  pA=p[1]; pB=p[2]; rA=p[3]; rBA=p[4]; rBa=p[5]
  if (param==1) { rBA <- p[5]/rA; rBa <- (p[4] - p[5]) /(1-rA)  }

  p=matrix(c(pA        ,     0     ,   0,
             0        ,    pB     ,   0,
             rA*(1-rBA), (1-rA)*rBa, rA*rBA), nrow=3, byrow=T)

  p=rbind(p,0); p=cbind(p,1-rowSums(p))

  seasn_end=seq(SPS,K,SPS)

  trans <- function(j, trustate, n, hst) {
    ntran = n * phi[trustate,]
    for (newtrustate in 1:4)
      cap(j+1,ntran[newtrustate],hst,newtrustate)
  }

  cap <- function(j,n,hst,trustate) {  #  determine how many sites with detections or
    if (n>0) {                      #  non-detections (recursively) each survey
      newhst=paste0(hst,c('1','2','3','0'))
      ndet=n*p[trustate,]

      if (j<K) {
        for (obsstate in 1:4) {
          if (ndet[obsstate]>0) {
            if (j %in% seasn_end) {
              trans(j, trustate, ndet[obsstate], newhst[obsstate])
            } else {
              cap(j+1,ndet[obsstate],newhst[obsstate],trustate)
            }
          }
        }
      } else {
        for (obsstate in 1:4)
          if (ndet[obsstate]>0) {
            e$xhst=c(e$xhst,newhst[obsstate])
            e$frq=c(e$frq,ndet[obsstate])
          }
      }
    }
  }
  occ=N*psi1; e=environment(); e$frq=NULL; e$xhst=NULL
  for (i in 1:4) cap(1,occ[i],'',i);   # generate det. histories for occupied sites
  tbl=table(e$xhst,e$frq); x=as.numeric(colnames(tbl))
  frq=apply(tbl,1,function(a)sum(a*x))
  hst=matrix(unlist(strsplit(names(frq),"")),ncol=K,byrow=T)
  names(frq)=NULL
  return(list(hst=hst, frq=frq))
}

genpresSD2sp <- function(N=100,K=4,
                         psi=c(.8,.3,.7),
                         p=c(.6,.66,.7,.4,.5),
                         gam=c(0,0,0),eps=c(0,0,0),
                         SPS=0, param=0,seed=0) {
  #' generate simulated data detection histories for 2-species models
  #'
  #' \code{genpresEV2sp} generates expected value detection history frequencies for
  #' single or multi-season 2-species models.
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy (psiA, psiBA, psiBa)
  #'@param p probability of detection, given occupied state
  #'@param gam prob. colonization (gamA, gamBA, gamBa)
  #'@param eps prob. extinction (epsA, epsBA, epsBa)
  #'@param SPS number of surveys per season
  #'@param param 0=default=psiA, psiBA, psiBa parm, 1=psiA,psiB,psiAB parm
  #'@param seed seed number for random number generation
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least 3^(K-1) detection-histories,
  #'          so if K>6, it may take a very long time to run.
  #'@seealso \code{\link{genpresEV}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}},  \code{\link{genpresEVms}}
  #'
  #'@examples
  #'\dontrun{
  #' # generate expected value data for 2-species single-season model
  #' x1=genpresSD2sp(N=100,K=4,
  #'             psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
  #'             p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
  #'             gam=c(0, 0, 0),             #  gamA, gamBA, gamBa
  #'             eps=c(0, 0, 0),             #  epsA, epsBA, epsBa
  #'             SPS=4)                      #  surveys per season
  #'
  #' # generate expected value data for 2-species multi-season model
  #' x2=genpresSD2sp(N=100,K=4,              # 100 sites, 4 surveys
  #'             psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
  #'             p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
  #'             gam=c(.1, .14, .17),        #  gamA, gamBA, gamBa
  #'             eps=c(.2, .22, .24),        #  epsA, epsBA, epsBa
  #'             SPS=2)                      #  surveys per season
  #'}
  if (seed>0) set.seed(seed)
  if (SPS==0) SPS=K
  psiA=psi[1]; psiBA=psi[2]; psiBa=psi[3]
  if (param==1) {
    # psiAB = psiA * psiBA               -> psiBA = psiAB/psiA
    # psiB = psiA*psiBA + (1-psiA)*psiBa -> psiBa = (psiB-psiA*psiBA)/(1-psiA)
    psiBA <- psi[3]/psiA; psiBa <- (psi[2] - psi[3]) /(1-psiA)
  }
  psi1=c(psiA*(1-psiBA), (1-psiA)*psiBa, psiA*psiBA)
  psi1=c(psi1,1-sum(psi1))

  if (length(gam)==3) gam[6]=gam[5]=gam[3]; gam[4]=gam[3]=gam[2]; gam[2]=gam[1]
  if (length(eps)==3) eps[6]=eps[5]=eps[3]; eps[4]=eps[3]=eps[2]; eps[2]=eps[1]

  gAB=gam[1]; gAb=gam[2]; gBAA=gam[3]; gBAa=gam[4]; gBaA=gam[6]; gBaa=gam[6]
  eAB=eps[1]; eAb=eps[2]; eBAA=eps[3]; eBAa=eps[4]; eBaA=eps[6]; eBaa=eps[6]

  #                  A0               0B               AB
  phi=matrix( c( (1-eAb)*(1-gBAA),     eAb*gBAa    ,   (1-eAb)*gBAA  , # A0
                 gAB*eBaA        , (1-gAB)*(1-eBaa),   gAB*(1-eBaA)  , # 0B
                 (1-eAB)*eBAA    ,   eAB*(1-eBAa)  , (1-eAB)*(1-eBAA), # AB
                 gAb*(1-gBaa)   , (1-gAb)*gBaa    ,  gAb*gBaA      ), # 00
              nrow=4,byrow=T)
  phi=cbind(phi,1-rowSums(phi))

  pA=p[1]; pB=p[2]; rA=p[3]; rBA=p[4]; rBa=p[5]
  if (param==1) { rBA <- p[5]/rA; rBa <- (p[4] - p[5]) /(1-rA)  }
  p=matrix(c(pA        ,     0     ,   0,
             0        ,    pB     ,   0,
             rA*(1-rBA), (1-rA)*rBa, rA*rBA), nrow=3, byrow=T)
  p=rbind(p,0); p=cbind(p,1-rowSums(p))

  seasn_end=NA; if (SPS < K) seasn_end = seq(SPS,K-1,SPS)

  occprobs=c(psiA*(1-psiBA), (1-psiA)*psiBa, psiA*psiBA, (1-psiA)*(1-psiBa))
  randocc <- rmultinom(N,1, occprobs)
  occ <- which(randocc>0, arr.ind=TRUE)[,1];

  det=matrix(NA,N,K);       occprobs=phi[occ,]; srvy=1

  for (srvy in 1:K) {
    detprobs=p[occ,]
    det[,srvy]=apply(detprobs,1,function(px) {
      rnddet<-rmultinom(n=1,size=1,prob=px)
      which(rnddet>0, arr.ind=TRUE)[,1]
    })

    if (srvy %in% seasn_end) {
      lastocc=occ
      occ=apply(occprobs,1,function(p) {
        randocc=rmultinom(1,1,prob=p)
        which(randocc>0, arr.ind=TRUE)[,1]
      })
    }
  }

  frq=table(apply(det,1,paste,collapse=''))
  hst=gsub('4','0',names(frq))
  i=order(hst); hst=hst[i]; frq=frq[i]
  hst=matrix(as.numeric(unlist(strsplit(hst,''))),ncol=K,byrow=T)
  names(frq)=NULL

  return(list(hst=hst, frq=frq))
}

genpresRPC <- function(N=100,K=4, lambda=2, r=.2,seed=0) {
  #' generate simulated data detection histories Royle point-count model
  #'
  #' \code{genpresEV2sp} generates expected value detection histories for
  #' Royle point-count model
  #'@param N number of sites
  #'@param K number of surveys
  #'@param lambda avg. number of individuals per site
  #'@param r probability of detection of an individual, given occupied state
  #'@param seed seed number for random number generation
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates histories with counts, instead of
  #'          just 0's and 1's.
  #'@seealso \code{\link{genpresEV}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}},  \code{\link{genpresEVms}}
  #'
  #'@examples
  #'\dontrun{
  #' # generate expected value data for 2-species single-season model
  #' x1=genpresSD2sp(N=100,K=4,
  #'             psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
  #'             p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
  #'             gam=c(0, 0, 0),             #  gamA, gamBA, gamBa
  #'             eps=c(0, 0, 0),             #  epsA, epsBA, epsBa
  #'             SPS=4)                      #  surveys per season
  #'
  #' # generate expected value data for 2-species multi-season model
  #' x2=genpresSD2sp(N=100,K=4,              # 100 sites, 4 surveys
  #'             psi=c(.8,.3,.7),            # psiA, psiBA, psiBa
  #'             p=c(.6,.66,.7,.4,.5),       #  pA, pB, rA, rBA, rBa
  #'             gam=c(.1, .14, .17),        #  gamA, gamBA, gamBa
  #'             eps=c(.2, .22, .24),        #  epsA, epsBA, epsBa
  #'             SPS=2)                      #  surveys per season
  #'}
  if (seed>0) set.seed(seed)
  pop=rpois(N,lambda) # simulate number of individuals per site

  count=NULL          # for each survey ...
  for (k in 1:K) {    #    simulate number of individuals detected
    count=cbind(count, rbinom(N, pop, p))
  }
}

genpresEVhet <- function(N=100,K=4,psi=.75,p=c(.4,.8),eps=.0,gam=.0,pi=0.3,SPS=0,opt=0) {
  #' generate expected value detection histories for model with heterogeneous detections
  #'
  #' \code{genpresEV} generates expected value detection history frequencies for
  #' single or multi-season heterogeneity models
  #'@param N number of sites
  #'@param K number of surveys
  #'@param psi probability of occupancy
  #'@param p probability of detection for each group
  #'@param eps probability of local extinction between seasons
  #'@param gam probability of local colonization between seasons
  #'@param SPS number of surveys per season
  #'@param opt option to restrict assignment of group membership to initial season
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least 2^K detection-histories, so if K>8,
  #'        it may take a very long time to run.
  #'        The "opt" argument controls how sites are assigned to the 2 groups.
  #'        By default (opt=0), sites can change groups each season, even if
  #'        the "pi" parameter is constant over seasons.  If opt=1, sites
  #'        remain in the same group over all seasons.
  #'
  #'@seealso \code{\link{genpresEVms}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}}
  #'@examples
  #'\dontrun{
  #'    # generate expected value data for single-season model with heterogeneous detections.
  #' hstfrq=genpresEVmshet(N=1000, K=4, psi=.8, p=c(.4,.8), pi=0.3)  # generate det. hists and freqs
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #' m1=occMod(data=EVpao, model=list(psi~1, p~GRP, pi~1), type="so.het")
  #' print_one_site_estimates(m1)
  #'
  #'    # generate expected value data for multi-season model with heterogeneous detections
  #' hstfrq=genpresEVhet(N=1000, K=8, psi=.8, p=c(.4,.8), gam=.2, eps=.25, pi=.3, SPS=2)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq, nsurveyseason=rep(4,2))    #  create pao file
  #' m2=occMod(data=EVpao, model=list(psi~1, gamma~1, epsilon~1, p~GRP, pi~1), type="do.het")
  #' print_one_site_estimates(m2)
  #'

  e=environment(); e$frq=e$xhst=NULL; if (SPS==0) SPS=K

  #                       a   b           A    B
  initstate=N*c((1-psi)*c(pi,1-pi), psi*c(pi,1-pi))

  #           a    b      A         B
  px=matrix(c(1,   1,   1-p[1] ,  1-p[2],  # not detected
              0,   0,    p[1]  ,   p[2]),  #  detected
            nrow=2, byrow=T)
  #                a               b                A             B
  phi=matrix(c(  pi*(1-gam),  (1-pi)*(1-gam)  ,   pi*gam ,   (1-pi)*gam,    #  a
                 pi*(1-gam),  (1-pi)*(1-gam)  ,   pi*gam ,   (1-pi)*gam,    #  b
                 pi*eps    ,  (1-pi)*eps      ,   pi*(1-eps),(1-pi)*(1-eps),#  A
                 pi*eps    ,  (1-pi)*eps      ,   pi*(1-eps),(1-pi)*(1-eps) #  B
  ),
  ncol=4, byrow=T)
  if (opt) {  #  if opt==1, group assigned in init state and doesn't change
              #   so, don't use pi in transitions between seasons.
    #                a          b           A         B
    phi=matrix(c(  (1-gam) ,     0      ,   gam    ,   0    ,    #  a
                   0       ,  (1-gam)   ,    0     ,   gam  ,    #  b
                   eps     ,     0      ,  (1-eps) ,   0    ,    #  A
                   0       ,    eps     ,    0     , (1-eps)     #  B
    ),  ncol=4, byrow=T)
  }
  srv <- function(k,n,hst, currentstate) {
    if (sum(n)>0) {
      newn = n %*% phi[currentstate,]
      for (i in 1:4) cap(k+1, newn[i],  hst, i) # generate det. histories for occupied sites
    }
  }

  cap <- function(j,n,hst, currentstate) {  #  determine how many sites with detections or
    ndet=n * px[,currentstate]   # number not det, det in A,B
    newhst=paste0(hst, c(0,1))
    if (j<K) {
      if ((j %% SPS)==0) {
        for (i in 1:2) srv(j, ndet[i], newhst[i], currentstate)
      } else {
        for (i in 1:2) cap(j+1, ndet[i], newhst[i], currentstate)
      }
    } else {
      p=parent.frame()
      e$xhst=c(e$xhst, newhst[1], newhst[2])
      e$frq=c(e$frq, ndet[1], ndet[2])
    }
  }
  #  states are: unoccA, unoccB, occA, occB where A,B are the 2 het. groups
  for (i in 1:4) cap(1,initstate[i], '', i) # generate det. histories
  s=unique(e$xhst);
  sumfrq=sapply(1:length(s),function(i) sum(e$frq[e$xhst==s[i]])) # summarize freqs by det. hist.
  i=which(sumfrq>0);   #  omit det. hists with frq=0
  hst=matrix(as.numeric(unlist(strsplit(s[i],''))),ncol=K,byrow=TRUE);
  colnames(hst)=paste(1:K)
  return(list(hst=hst,frq=sumfrq))
}

LRTpower <- function(ch,df=1,alf=.05){
  #' compute expected power using non-central chi-square function
  #'
  #' \code{LRTpower} compute expected power using non-central chi-square function
  #' where power is defined as the probability of rejecting the null hypothesis
  #' when it is false.
  #'
  #' A description of an analytical-numerical approximation
  #' approach can be found in:
  #'
  #' \emph{Burnham, K.P., D.R. Anderson, G.C. White, C. Brownie and K.H. Pollock. 1987.
  #' Design and Analysis Methods for Fish Survival Experiments Based on
  #' Release-Recapture. Am. Fish. Soc. Monogr. 5, pages 216, 289-295.}
  #'
  #' The basic idea: Given the alternative model, Ha, is true, what is the
  #' probability that the likelihood-ratio test will reject the null model, Ho? If
  #' Ho is true, the power of the test should equal the significance level alfa.
  #'
  #' Data should be generated by computing expected values under the alternative
  #' model for specified sample sizes, compute estimates under the alternative and
  #' null models, then compute a likelihood-ratio test between models. The
  #' likelihood-ratio test will give you the chi-square value and degrees of freedom needed
  #' for the power analysis.
  #'
  #' Specifically, the chi-square value approximates the noncentrality parameter of a
  #' noncentral chi-square distribution, from which power is directly obtained.
  #'
  #'@param ch chi-square value from likelihood-ratio test
  #'@param df number of degrees of freedom
  #'@param alf significance level (default=0.05)
  #'@export
  #'@return power = prob. to reject Ho
  #'
  #'@examples
  #'\dontrun{
  #'    # compute power of likelihood-ratio test using default alfa=0.05
  #'# power=LRTpower(chisq1-chisq2,npar1-npar2)
  #'}
  1-pchisq(qchisq(1-alf,df),df,ch)
}

