# occMod_DO_het.R
occMod_DO_het<-function(psi=call(),psi.cov=data$unitcov,
               p=call(),p.cov=cbind(data$unitcov,data$survcov), pi=call(),
               gamma=call(), gamma.cov=NULL, epsilon=call(), epsilon.cov=NULL,
               modname=NULL,outfile,model=100,fixed=NULL,initvals=NULL,data,conf,miscopts){

#psi=p=gamma=epsilon=pi=~1; data=pao; modname='asdf'; outfile='asdf.out'; initvals=c(1,1,-1,-1,-1)
#psi.cov=p.cov=gamma.cov=epsilon.cov=fixed=NULL; miscopts='maxfn=32000'
  #' Fit static occupancy/single season model
  #'
  #' This is not intended for direct use, but instead the \code{\link{occMod}} function should be used with \code{type="so"}.
  #'
  #'@param psi the right-hand side of the formula for the model to fit for occupancy probability.
  #'@param psi.cov a data frame containing the unit-specific covariates to use for the occupancy
  #'component of the model.
  #'@param p the right-hand side of the formula for the model to fit for detection probability.
  #'@param p.cov a data frame containing the survey-specific covariates to use for the detection
  #'component of the model.
  #'@param pi hetergeneity pi parameter (prop sites in group 1).
  #'@param gamma the right-hand side of the formula for the model to fit for colonization probability in each season.
  #'@param gamma.cov a data frame containing the unit-specific covariates to use for the colonization component of the model, with number of rows = \code{data$nunits*(data$nseasons-1)}.
  #'@param epsilon the right-hand side of the formula for the model to fit for extinction probability in each season.
  #'@param epsilon.cov a data frame containing the unit-specific covariates to use for the extinction component of the model, with number of rows = \code{data$nunits*(data$nseasons-1)}.
  #'@param modname (optional) a string containing the model name
  #'@param outfile name for output file (use outfile='modname') for outfile named via model name
  #'@param model the PRESENCE model code. DO NOT CHANGE.
  #'@param fixed a single-column matrix containing values for real parameters to be fixed at.
  #'\code{rownnames(fixed)} should contain the index of the real parameters to be fixed.
  #'@param initvals initial values for the beta parameters at which PRESENCE begins the optimisation.
  #'The default values in PRESENCE is 0.
  #'@param data the \code{pao} data object containing the detection data and other information.
  #'@param conf level for confidence interval (may be vector valued).
  #'@param miscopts char vector of options to send to Presence (eg., maxfn=32000)- last element must be ''.  #'
  #'@return returns a list of class \code{"occMod"} and \code{"so"}.
  #'
  #'\code{occMod$beta} contains the objects:
  #'  #'\item{psi}{estimated logistic regression coefficients and standard errors for probability of occurrence.}
  #'\item{psi.VC}{variance-covariance matrix for logistic regression coefficients for probability of occurrence.}
  #'\item{p}{estimated logistic regression coefficients and standard errors for probability of detection.}
  #'\item{p.VC}{variance-covariance matrix for logistic regression coefficients for probability of detection.}
  #'\item{VC}{the full variance-covariance matrix for all logistic regression coefficients.}
  #'
  #'\code{occMod$real} contains the objects:
  #'\item{psi}{estimated probabilities of occurrence for each sampling unit, along with standard
  #'errors and limits of 95\% confidence interval.}
  #'\item{p}{estimated probabilities of detection for each survey, along with standard errors and
  #'limits of 95\% confidence interval.}
  #'
  #'\code{occMod$derived} contains the objects:
  #'\item{psi_c}{estimated probabilities of occurrence given the detection history for each sampling
  #'unit, along with standard errors and limits of 95\% confidence interval. Will be \code{=1} for
  #'any unit where the species was detected at least once.}
  #'
  #'@author Darryl MacKenzie
  #'
  #'@seealso \code{\link{occMod}}, \code{pao}
  #' @examples
  #'
  #' # load a csv data file
  #' filename<-system.file("extdata/skinks.csv",package="RPresence")
  #' csv<-read.csv(filename,as.is=T,header=F)
  #' ##
  #' nsites=nrow(csv); nsrvys=ncol(csv)  #  set number of sites,surveys from det. data
  #' dethist=matrix(as.integer(unlist(csv)),nrow=nsites)
  #' ##          create input "pao" object, for use with occMod function
  #' data=createPao(dethist, title="single-sp, detection heterogeneity example")
  #'
  #' mod1<-occMod(model=list(psi~1,gamma~1,epsilon~1,p~1,pi~1),data=data,type="do.het")
  #'

  options("na.action"="na.pass")
  n=data$nunits; k=data$nsurveys; ns=data$nseasons
  if (is.null(psi.cov))
    psi_mat=model.matrix(psi,data.frame(rep(1,n))) else
    psi_mat=model.matrix(psi,psi.cov)

  if (is.null(p.cov)) p.cov=data.frame(SURVEY=rep(1,n*k*2))
  if (nrow(p.cov)==k*n) p.cov=rbind(p.cov,p.cov)
  p.cov$SEASON=1; ptr=0; pnames=NULL
  for (i in 1:ns)
    for (j in 1:data$nsurveyseason[i]) {
      p.cov$SEASON[ptr+1:n]=paste(i)
      p.cov$SURVEY[ptr+1:n]=paste(j)
      pnames=c(pnames,paste0('p1(',i,'-',j,')',data$unitnames))
      ptr=ptr+n
    }
  pnames=c(pnames,gsub('p1','p2',pnames))
  p.cov$SEASON[ptr+1:ptr]=p.cov$SEASON[1:ptr]
  p.cov$SURVEY[ptr+1:ptr]=p.cov$SURVEY[1:ptr]
  p.cov$GRP='A'; p.cov$GRP[ptr+1:ptr]="B"
  p_mat<-model.matrix(p,p.cov)

  #  only allow constant or season-specific pi's - no covariates
  pi_mat=model.matrix(pi, data.frame(SEASON=paste(rep(1:data$nseasons))))
  if (pi == ~0) pi_mat=model.matrix(~1, data.frame(SEASON=paste(rep(1:data$nseasons))))
  ## create and output temporary pao file
  temp.pao=data
  temp.pao$nunitcov=ncol(psi_mat); temp.pao$unitcov=as.data.frame(psi_mat)
  temp.pao$nsurvcov=ncol(p_mat)
  temp.pao$survcov=as.data.frame(p_mat)
  temp.pao$paoname=ifelse(is.null(data$paoname),"paodata.pao",data$paoname)

  colnames(temp.pao$unitcov)=gsub("[(]Intercept[)]","int",colnames(temp.pao$unitcov))
  colnames(temp.pao$unitcov)=paste("psi.",colnames(temp.pao$unitcov),sep="")
  colnames(temp.pao$survcov)=gsub("[(]Intercept[)]","int",colnames(temp.pao$survcov))
  colnames(temp.pao$survcov)=paste("p.",colnames(temp.pao$survcov),sep="")

  ## create design matrices file
  psi.dm=matrix(colnames(temp.pao$unitcov),1,temp.pao$nunitcov)
  psi.dm.col=ncol(psi.dm)
  rownames(psi.dm)="psi"
  colnames(psi.dm)=paste("a",1:temp.pao$nunitcov,sep="")

  p.dm=matrix(rep(colnames(temp.pao$survcov),k*2),ncol=temp.pao$nsurvcov,byrow=TRUE);
  rownames(p.dm)=c(paste0("p1",1:k), paste0("p2",1:k))
  colnames(p.dm)=paste0("d",1:ncol(p.dm))
  if (pi == ~0) {
    pi.dm=matrix(1, nrow=1, ncol=1, dimnames=list('pi', 'e1'))
  } else {
    pi.dm=matrix(pi_mat, nrow(pi_mat), ncol(pi_mat))
    rownames(pi.dm)=paste0('pi',1:data$nseasons)
    colnames(pi.dm)=paste0('e', 1:ncol(pi.dm))
  }

  # delete last season for gam/eps covs
  SEASON=as.factor(rep(1:(data$nseasons-1),each=data$nunits));
  if(is.null(gamma.cov)) {
    gamma.cov=  data.frame(SEASON=SEASON)
  } else {
    s=names(gamma.cov); gamma.cov=data.frame(gamma.cov[1:(data$nunits*(data$nseasons-1)),]);
    names(gamma.cov)=s; gamma.cov$SEASON=SEASON
  }
  if(is.null(epsilon.cov)) {
    epsilon.cov=data.frame(SEASON=SEASON)
  } else {
    s=names(epsilon.cov); epsilon.cov=data.frame(epsilon.cov[1:(data$nunits*(data$nseasons-1)),]);
    names(epsilon.cov)=s; epsilon.cov$SEASON=SEASON
  }
  gamma_mat=  model.matrix(gamma,gamma.cov[1:(data$nunits*(data$nseasons-1)),])
  epsilon_mat=model.matrix(epsilon,epsilon.cov[1:(data$nunits*(data$nseasons-1)),])

  ## create and output temporary pao file
  ### bind each block of psi covariates to unitcov
  temp.cov=psi_mat
  for(ii in 1:(data$nseasons-1)){
    temp.cov=cbind(temp.cov, gamma_mat[(ii-1)*data$nunits+1:data$nunits,],
                   epsilon_mat[(ii-1)*data$nunits+1:data$nunits,])
  }

  s=c(paste0("psi.",colnames(psi_mat)),
      paste0(rep(c(paste0("gamma.",  colnames(gamma_mat)),
                   paste0("epsilon.",colnames(epsilon_mat))),data$nseasons-1),
             rep(1:(data$nseasons-1),each=(ncol(gamma_mat)+ncol(epsilon_mat)))))
  colnames(temp.cov)=gsub("(Intercept)","int",s,fixed=T)
  temp.pao$unitcov=as.data.frame(temp.cov); temp.pao$nunitcov=ncol(temp.cov)

  index=grep("gamma.",colnames(temp.pao$unitcov),fixed=TRUE)
  gamma.dm=matrix(colnames(temp.pao$unitcov)[index], (temp.pao$nseasons-1),ncol(gamma_mat),byrow=TRUE)
  rownames(gamma.dm)=paste("gam(",1:(temp.pao$nseasons-1),")",sep="")
  colnames(gamma.dm)=paste("b",1:ncol(gamma_mat),sep="")

  index=grep("epsilon.",colnames(temp.pao$unitcov),fixed=TRUE)
  epsilon.dm=matrix(colnames(temp.pao$unitcov)[index], (temp.pao$nseasons-1),ncol(epsilon_mat),byrow=TRUE)
  rownames(epsilon.dm)=paste("eps(",1:(temp.pao$nseasons-1),")",sep="")
  colnames(epsilon.dm)=paste("c",1:ncol(epsilon_mat),sep="")

  psi.dm=simplify_dm(psi.dm, temp.pao);
  p.dm=simplify_dm(p.dm, temp.pao, het=1)
  gam.dm=simplify_dm(gamma.dm, temp.pao);
  eps.dm=simplify_dm(epsilon.dm, temp.pao)

  if(!is.null(fixed)){
    rnames=unlist(lapply(list(psi.dm,p.dm,pi.dm,eps.dm, gam.dm),rownames))
    fixed$idx<-match(fixed$param,rnames)
    if (sum(is.na(fixed$idx))>0) cat('fixed names must be in ',rnames,'\n')
  }
  temp.pao$paoname="savepao.pao"
  rv=runPresence(temp.pao,list(psi.dm,gam.dm, eps.dm, p.dm,pi.dm,NULL),
                model,modname=modname,fixed=fixed,initvals=initvals,
                outfile=outfile,miscopts=miscopts)

  ### extract results from output file
  v=readLines(outfile)

  warn.conv=check_conv_warn(v); warn.VC=check_VC_warn(v)   ##### check for warnings

  ### get PRESENCE version
  i=grep("Version",v,fixed=TRUE)[1]; version=gsub(".+Version ","",v[i])

  ## find AIC value for each model
  i=grep("-2log.likelihood",v);
  neg2loglike=as.numeric(gsub(".+= ","",v[i]));
  aic=as.numeric(gsub(".+= ","",v[i+1]))
  v=v[-1:-i]

  ### get Untransformed coefficients (Beta's)
  i=grep("^A.+psi.+ : ",v); psi.coeff=getbetas(v[i],psi.dm)
  ib=grep("^B.+gam.+ : ",v); gam.coeff=getbetas(v[ib],psi.dm)
  ic=grep("^C.+eps.+ : ",v); eps.coeff=getbetas(v[ic],psi.dm)
  id=grep("^D.+ : ",v); p.coeff=getbetas(v[id],p.dm)
  ie=grep("^E.+ : ",v); pi.coeff=getbetas(v[ie],pi.dm)

  ### get_VC matrix
  ii=grep("^[A-F][0-9].+ : ",v); names=gsub(" .+","",v[ii]); npar=length(names)
  instns=grep("Variance-Covariance Matrix of",v,fixed=TRUE)
  VC=get_VC(v=v,offset=instns+1,n.coeff=npar,cov.names=names)

  psi.VC=as.matrix(VC[1:length(i), 1:length(i)]); j=length(i)
  gam.VC=as.matrix(VC[j+1:length(ib), j+1:length(ib)]); j=j+length(ib)
  eps.VC=as.matrix(VC[j+1:length(ic), j+1:length(ic)]); j=j+length(ic)
  p.VC  =as.matrix(VC[j+1:length(id), j+1:length(id)]); j=j+length(id)
  pi.VC =as.matrix(VC[j+1:length(ie), j+1:length(ie)])

  psi.est=gam.est=eps.est=p.est=psi_c.est=pi.est=NULL
  VCoutopt=miscopts[1]
  if (! (VCoutopt %in% c("nose","betavc","noreal"))) {  ## return real parameters
    ## Calc real parameter estimates
    psi.est=calc_real(cov=psi_mat,coeff=psi.coeff$est,VC=psi.VC, conf=conf,
              rownames=paste0('psi_',temp.pao$unitnames))
    gam.est=calc_real(cov=gamma_mat,coeff=gam.coeff$est,VC=gam.VC, conf=conf,
              rownames=paste0('gam(',rep(1:(ns-1),each=n),')',rep(temp.pao$unitnames,ns-1)))
    eps.est=calc_real(cov=epsilon_mat,coeff=eps.coeff$est,VC=eps.VC, conf=conf,
              rownames=paste0('eps(',rep(1:(ns-1),each=n),')',rep(temp.pao$unitnames,ns-1)))
    s=paste0('p1(',rep(1:k,each=n),')',rep(temp.pao$unitnames,k))
    s=c(s,paste0('p2(',rep(1:k,each=n),')',rep(temp.pao$unitnames,k)))

    p.est=calc_real(cov=p_mat,coeff=p.coeff$est,VC=p.VC,conf=conf, fixed=fixed,
              rownames=pnames)

    pi.est=calc_real(cov=pi_mat,coeff=pi.coeff$est,VC=pi.VC,conf=conf,
                     rownames=paste0('pi_',1:nrow(pi_mat)),fixed=fixed)

    nk=n*k; ii=which(rowSums(temp.pao$det.data,na.rm=TRUE)>0)
    calc_psi_c <- function(b) {
      psiest = plogis(psi_mat %*% b[1:length(psi.coeff$est)])
      pest=plogis(p_mat %*% b[length(psi.coeff$est)+1:length(p.coeff$est)])
      piest=plogis(pi_mat %*% b[length(b)])
      qstar1=apply(1-matrix(pest[1:nk],ncol=k),1,prod)
      qstar2=apply(1-matrix(pest[nk+1:nk],ncol=k),1,prod)
      qstar=piest[1]*qstar1+(1-piest[1])*qstar2
      rv=psiest*qstar/(psiest*qstar+1-psiest)
      rv[ii]=1
      return(rv)
    }
    b=c(psi.coeff$est,gam.coeff$est,eps.coeff$est,p.coeff$est,pi.coeff$est)
    psi_c.est=calc_psi_c(b); grad=matrix(0,length(psi_c.est), length(b))
    for (i in 1:length(b)) {
      b[i]=b[i]+.1e-9
      psi_c_tmp = calc_psi_c(b)
      grad[,i] = (psi_c_tmp - psi_c.est) /.1e-9
      b[i]=b[i] - .1e-9
    }
    newvc = grad %*% VC %*% t(grad)
    se=sqrt(diag(newvc)); se=se/psi_c.est/(1-psi_c.est)
    ci1=plogis(qlogis(psi_c.est)-1.96*se); ci2=plogis(qlogis(psi_c.est)+1.96*se)
    psi_c = cbind(psi_c.est, se, ci1, ci2); rownames(psi_c)=temp.pao$unitnames
  } # end if(!noReal)
  #  get gof results if desired
  gof=list(chat=NA,TS=NA,tbl=NULL,bs=NA)

  result=list(modname=modname, model=list(psi=psi,p=p),
              dmat=list(psi=psi.dm,gam.dm,eps.dm,p=p.dm,pi=pi.dm),
               data=temp.pao, outfile=outfile, neg2loglike=neg2loglike, npar=npar, aic=aic,
               beta=list(psi=as.data.frame(psi.coeff),psi.VC=psi.VC,
                         gam=as.data.frame(gam.coeff),gam.VC=gam.VC,
                         eps=as.data.frame(eps.coeff),eps.VC=eps.VC,
                         p=as.data.frame(p.coeff),p.VC=p.VC, VC=VC),
               real=list(psi=as.data.frame(psi.est), gamma=data.frame(gam.est),
                         epsilon=data.frame(eps.est), p=as.data.frame(p.est),
                         pi=data.frame(pi.est)),
               derived=list(psi_c=psi_c), gof=gof,
               warnings=list(conv=warn.conv,VC=warn.VC),
               version=list(PRESENCE=version,RPresence=packageVersion("RPresence")))
  class(result)=c("occMod","so")
  return(result)
}
