#########################################
occMod_DO_ms2<-function(psi=call(),psi.cov=data$unitcov, Cpsi=call(), Cpsi.cov=NULL,
                      R=call(),R.cov=data$unitcov, CR=call(), CR.cov=NULL,
                      p=call(),p.cov=data$survcov,
                      delta=call(),delta.cov=data$survcov,
                      modname=NULL, outfile,
                      model=6100,fixed=NULL,initvals=NULL,data,conf,miscopts){
#' Fit multi-state dynamic occupancy (multi-season) occupancy model using the second (conditional binomial) parameterisation in PRESENCE.
#'
#' This is not intended for direct use, but instead the \code{\link{occMod}} function should be used with \code{type="do.ms.2"}.
#'
#' @param psi the right-hand side of the formula for the model to fit for initial occupancy probability.
#' @param psi.cov a data frame containing the unit-specific covariates to use for the occupancy component of the model.
#' @param Cpsi the right-hand side of the formula for the model to fit for occupancy probability, both first year and dynamic occupancy.
#' The terms \code{SEASON}, \code{DYN} and \code{PREV_STATE} can each be used without having been defined in \code{psi.cov}.
#' \code{SEASON} allows for a seasonal effect, \code{DYN} enables dynamic occupancy probabilities to be different from first year occupancy,
#' and \code{PREV_STATE} allows the probability to be different depending on the state in the previous season.
#' @param Cpsi.cov a data frame containing the unit-specific covariates to use for the occupancy component of the model.
#' @param R the right-hand side of the formula for the model to fit for the probability of being in the second state, conditional on the unit being occupied.
#' @param R.cov a data frame containing the unit-specific covariates to use for the probability of being in the second state, conditional on the unit being occupied part of the model.
#' @param CR the right-hand side of the formula for the model to fit for the probability of being in the second state, conditional on the unit being occupied.
#' The same built-in covariates, \code{SEASON}, \code{DYN} and \code{PREV_STATE} can be used for CR.
#' @param CR.cov a data frame containing the unit-specific covariates to use for the probability of being in the second state, conditional on the unit being occupied part of the model.
#' @param p the right-hand side of the formula for the model to fit for detection probability.
#' The terms \code{SEASON} and \code{STATE} can each be used without having been defined in \code{p.cov}.
#' \code{SEASON} allows for a seasonal effect, and \code{STATE} allows the probability of detection to be different depending on the state in the current season.
#' @param p.cov a data frame containing the survey-specific covariates to use for the detection component of the model.
#' @param delta the right-hand side of the formula for the model to fit for the probability of detecting the second state in a survey,
#' conditional on the species being detected in the survey. The terms \code{SEASON} and \code{STATE} can each be used as above to the same effect.
#' @param delta.cov a data frame containing the survey-specific covariates to use for the probability of
#' detecting the second state in a survey, conditional on the species being detected in the survey.
#' @param modname (optional) a string containing the model name
#' @param outfile name for output file (use outfile='modname') for outfile named via model name
#' @param model the PRESENCE model code. DO NOT CHANGE.
#' @param fixed a single-column matrix containing values for real parameters to be fixed at. \code{rownnames(fixed)} should contain the index of the real parameters to be fixed.
#' @param initvals initial values for the beta parameters at which PRESENCE begins the optimisation. The default values in PRESENCE is 0.
#' @param data the \code{pao} data object containing the detection data and other information.
#' @param conf level for confidence interval (may be vector valued).
#' @param miscopts char vector of options to pass to Presence... last element must be null string.
#' @return returns a list of class \code{"occMod"} and \code{"do.1"}.
#'
#' \code{occMod$beta} contains the objects:
#'  \item{psi}{estimated logistic regression coefficients and standard errors for probability of occurrence.}
#'  \item{psi.VC}{variance-covariance matrix for psi.}
#'  \item{R}{estimated logistic regression coefficients and standard errors for probability of breeding (state=2), givven occupancy.}
#'  \item{R.VC}{variance-covariance matrix for R..}
#'  \item{p}{estimated logistic regression coefficients and standard errors for probability of detection.}
#'  \item{p.VC}{variance-covariance matrix for p.}
#'  \item{VC}{the full variance-covariance matrix for all logistic regression coefficients.}
#'
#'  \code{occMod$real} contains the objects:
#'  \item{psi}{estimated probabilities of occurrence for each sampling unit, along with standard errors and limits of 95\% confidence interval.}
#'  \item{p}{estimated probabilities of detection for each survey, along with standard errors and limits of 95\% confidence interval.}
#'
#' @examples
#'
#' cat('# --- SINGLE-SEASON MULTI-STATE EXAMPLE\n')
#' cat('3 OCCUPANCY STATES, PSI-R PARAMETEIZATION) (PG 231 IN BOOK) ----\n')
#'
#'  #    read detection history data from csv file...
#'  dethist<-read.csv(system.file(
#'  "extdata/cal_owl_multistate_data.csv",package="RPresence"),na.strings=".",
#'                    colClasses="numeric")[,-1]
#'nsites=nrow(dethist) #  set number of sites,surveys from det. history data
#'nsrvys=ncol(dethist)
#'frq=rep(1,nsites)
#'#  create survey covariate to categorize surveys 1-2 as 1 period
#'#                                    and surveys 3-5 as another period
#'  #    Since it's a survey covariate, it will be a NxT matrix (N=nsites, T=nsurveys)
#'#    The covariate matrix would be:
#'#         1 2 3 4 5
#'#  site1 [1 1 2 2 2]
#'#  site2 [1 1 2 2 2]
#'#   :    [: : : : :]
#'#  siteN [1 1 2 2 2]
#'#    Filling in by cols means we repeat "1" N times, then "1" N times,
#'#                  then "2" N times, then "2" N times, then "2" N times.
#'#    We save it as a data frame (without the matrix dimensions).
#'cov2=data.frame(PER=as.factor(c(rep(1,2*nsites),rep(2,3*nsites))))
#'
#'#          create input "pao" object, for use with occMod function
#'data=createPao(data=dethist,frq=frq,survcov=cov2,title="Cal Owl example")
#'
#'xmods<-occMod(model=list(psi~1,R~1,p~STATE, delta~PER),data=data,type="do.ms.2",outfile='modname')
#'
#'x=rbind(unique(xmods$real$psi[,1:2]),unique(xmods$real$R[,1:2]),
#'          unique(xmods$real$p[,1:2]),unique(xmods$real$delta[,1:2]))
#'x$est=round(x$est,2); rownames(x)=c('psi','R','p1','p2','dlta1','dlta2')
#'xmods_chk=round(x,3)
#'print(xmods_chk)
#'
#'cat('# --- MULTI-SEASON MULTI-STATE EXAMPLE (3 OCCUPANCY STATES, PSI-R PARAMETEIZATION) ----\n')
#'
#'  hstfrq=genpresEVms(N=100,K=4,psi=.75,R=.8, Cpsi=c(.66,.1,.15),CR=c(.72,.77,0),
#'                   p1=.4,p2=.3,dlta=.6,nstates=3,SPS=2)
#'pao=createPao(data=hstfrq$hst,frq=hstfrq$frq,title="exp. value data",nsurveyseason=c(2,2))
#'
#'m1=occMod(list(psi~1,Cpsi~STATE,R~1,CR~STATE,p~STATE,delta~1),
#'          data=pao,type='do.ms.2',outfile='modname')
#'
#'cat('Occupancy estimates:\n'); print(unique(m1$real$psi))
#'cat('Occupancy-state2 estimates:\n'); print(unique(m1$real$R))
#'cat('Detection estimates:\n'); print(unique(m1$real$p))
#'cat('Detection-state2 estimates:\n'); print(unique(m1$real$delta))
#'
#' @author Darryl MacKenzie, Jim Hines
#' @seealso \code{\link{occMod}}

  if(!("pao"%in%class(data))) { print("data must be a pao object!!");  return(NULL)  }
  options("na.action"="na.pass")

  nunits=data$nunits; nsrvys=data$nsurveys; ns=data$nseasons; ns1=ns-1; nsrvyseasn=data$nsurveyseason

  rep3x <- function(df) {    #  repeat season covariate for Cpsi 3 times...
    nc=ncol(df); cnames=names(df); rv=list()
    for (j in 1:nc) {                                       # for each covariate...
       x1=matrix(unlist(df[,j]),nrow=nunits); x1a=NULL      #    reshape into matrix with nsites rows
       for (i in 1:ncol(x1)) x1a=c(x1a,rep(x1[,i],3))       #    replicate each value 3 times and save as vector
       rv[[j]]=x1a                                          #    save vector as jth list element
    }
    names(rv)=cnames                                        # assign names from orig names of data frame
    return(data.frame(rv))                                  # convert to data frame
  }
  if (ns>1) {
    if (nrow(Cpsi.cov)<1) Cpsi.cov=data.frame(STATE=rep(rep(c('0','1','2'),each=nunits),each=ns1),stringsAsFactors=T)
    if (nrow(CR.cov)<1) CR.cov=data.frame(STATE=rep(rep(c('0','1','2'),each=nunits),each=ns1),stringsAsFactors=T)

    if (!is.null(Cpsi.cov))
      if (nrow(Cpsi.cov)==(nunits*ns1)) Cpsi.cov=rep3x(Cpsi.cov)
      if (!is.null(CR.cov))
        if (nrow(CR.cov)==(nunits*ns1)) CR.cov=rep3x(CR.cov)
  }
  if (!is.null(p.cov))    #  if p.cov contains > 0 covariates...
    if (nrow(p.cov)==(nunits*nsrvys))  # if only defined for 1 state p
      p.cov=rbind(p.cov,p.cov)

  #### create season-specific covariate table for psi and R

  if (ns>1) {
     if (is.null(Cpsi.cov$SEASON)) Cpsi.cov$SEASON=as.character(rep(rep(1:ns1,each=3),each=nunits))
     if (is.null(Cpsi.cov$STATE)) Cpsi.cov$STATE=as.character(rep(rep(0:2,ns1),each=nunits))
     if (is.null(Cpsi.cov$ALLDIFF)) Cpsi.cov$ALLDIFF=as.character(rep(1:(ns1*3),each=nunits))
     if (is.null(CR.cov$SEASON)) CR.cov$SEASON=as.character(rep(rep(1:ns1,each=3),each=nunits))
     if (is.null(CR.cov$STATE)) CR.cov$STATE=as.character(rep(rep(0:2,ns1),each=nunits))
     if (is.null(CR.cov$ALLDIFF)) CR.cov$ALLDIFF=as.character(rep(1:(ns1*3),each=nunits))
  }
  if (is.list(Cpsi.cov)) Cpsi.cov=data.frame(Cpsi.cov)
  if (is.list(CR.cov)) CR.cov=data.frame(CR.cov)
  if (is.null(psi.cov)) psi.cov=data.frame(x=rep(1,nunits))
  if (is.null(R.cov)) R.cov=data.frame(x=rep(1,nunits))

                  ## combine unit and survey specific covariates
  p.cov$SSURVEY=as.factor(rep(unlist(sapply(nsrvyseasn,function(i) rep(1:i,each=nunits))),2))
  if (is.null(p.cov$SEASON)) p.cov$SEASON=as.factor(rep(unlist(sapply(1:ns,function(i) rep(i,nsrvyseasn[i]*nunits))),2))
  if (is.null(p.cov$STATE))  p.cov$STATE =as.factor(rep(0:1,each=nsrvys*nunits))
  if (is.null(p.cov$ALLDIFF)) p.cov$ALLDIFF=as.factor(1:length(p.cov$SURVEY))
  if (is.null(delta.cov$SURVEY)) delta.cov$SURVEY=as.factor(unlist(sapply(nsrvyseasn,function(i) 1:i)))
  if (is.null(delta.cov$SEASON)) delta.cov$SEASON=as.factor(unlist(sapply(1:ns,function(i) rep(i,nsrvyseasn[i]))))
  if (is.null(delta.cov$ALLDIFF)) delta.cov$ALLDIFF=as.factor(1:length(delta.cov$SURVEY))
  if (is.list(p.cov)) p.cov=data.frame(p.cov)
  if (is.list(delta.cov)) delta.cov=data.frame(delta.cov)
  ##################
  psi_mat<-model.matrix(psi,psi.cov);
  R_mat  <-model.matrix(R,R.cov)    ;
  p_mat  <-model.matrix(p,p.cov)    ; delta_mat<-model.matrix(delta,delta.cov);
  colnames(psi_mat)=gsub('(Intercept)','_Int',colnames(psi_mat),fixed=T)
  colnames(R_mat)=gsub('(Intercept)','_Int',colnames(R_mat),fixed=T)
  colnames(p_mat)=gsub('(Intercept)','_Int',colnames(p_mat),fixed=T)
  colnames(delta_mat)=gsub('(Intercept)','_Int',colnames(delta_mat),fixed=T)
  if (ns>1) {
    Cpsi_mat <-model.matrix(Cpsi,Cpsi.cov);   CR_mat  <-model.matrix(CR,CR.cov)
    colnames(CR_mat)=gsub('(Intercept)','_Int',colnames(CR_mat),fixed=T)
    colnames(Cpsi_mat)=gsub('(Intercept)','_Int',colnames(Cpsi_mat),fixed=T)
  }
  ## create and output temporary pao file

  ### bind each block of psi covariates to unitcov
  temp.unitcov=matrix(c(psi_mat,R_mat),nrow=nunits);
  s=c(paste0('psi0',colnames(psi_mat)),paste0('R',colnames(R_mat)))
  colnames(temp.unitcov)=s

  ### bind each block of p and delta covariates to cov
  n2=nrow(p_mat)/2; temp.survcov=matrix(c(delta_mat,p_mat[1:n2,],p_mat[n2+1:n2,]),nrow=nunits*nsrvys)
  colnames(temp.survcov)=c(paste0(paste0('delta.'),colnames(delta_mat)),paste0(paste0('p1.'),colnames(p_mat)), paste0(paste0('p2.'),colnames(p_mat)))

  ##  next, we'll make the seasonal covariates into survey covariates...
  if (ns>1) {
    temp.seasncov=cbind(matrix(Cpsi_mat,nrow=nunits*ns1),matrix(CR_mat,nrow=nunits*ns1))
    temp.seasncov=cbind(matrix(Cpsi_mat,nrow=nunits),matrix(CR_mat,nrow=nunits))
    s=paste0(rep(paste0('Cpsi',0:2),ns1*ncol(Cpsi_mat)),rep(colnames(Cpsi_mat),each=3*ns1))
    s=c(s,paste0(rep(paste0('CR',0:2),ns1*ncol(CR_mat)),rep(colnames(CR_mat),each=3*ns1)))
    colnames(temp.seasncov)=paste0(s,'_',rep(1:ns1,each=3))
    temp.unitcov=cbind(temp.unitcov,temp.seasncov)
  }
  ##  next, combine season and survey covariates into survey covariate
  temp.cov=data.frame(temp.survcov)

  temp.pao=data; temp.pao$unitcov=temp.unitcov; temp.pao$nunitcov=ncol(temp.unitcov)
  temp.pao$survcov=temp.cov; temp.pao$nsurvcov=ncol(temp.cov)
  temp.pao$paoname=ifelse(is.null(data$paoname), "paodata.pao", data$paoname)

  ## create design matrices and run PRESENCE
  temp=grep("^psi",colnames(temp.pao$unitcov),value=T); psi.dm=matrix(temp,ncol=ncol(psi_mat),byrow=F)
  psi.dm=simplify_dm(psi.dm,temp.pao); rownames(psi.dm)='psi'
  temp=grep("^R",    colnames(temp.pao$unitcov),value=T);   R.dm=matrix(temp,ncol=ncol(R_mat),byrow=F)
  R.dm=simplify_dm(R.dm,temp.pao); rownames(R.dm)='R'

  x=diagbind(psi.dm,R.dm)
  if (ns>1) {
    temp=grep("^Cpsi",colnames(temp.pao$unitcov),value=T);
    Cpsi.dm =matrix(temp,nrow=ns1*3, byrow=FALSE)
    Cpsi.dm=simplify_dm(Cpsi.dm,temp.pao); rownames(Cpsi.dm)=paste0('Cpsi',rep(0:2,ns1),'(',rep(1:ns1,each=3),')')
    temp=grep("^CR",    colnames(temp.pao$unitcov),value=T);
    CR.dm=matrix(temp,nrow=ns1*3, byrow=FALSE)
    CR.dm=simplify_dm(CR.dm,temp.pao); rownames(CR.dm)=paste0('CR',rep(0:2,ns1),'(',rep(1:ns1,each=3),')')
    x=diagbind(diagbind(psi.dm,Cpsi.dm),diagbind(R.dm,CR.dm))
  }
  colnames(x)=paste0('a',1:ncol(x))
  psi.dm=x

  temp=grep("delta.",colnames(temp.pao$survcov),fixed=TRUE,value=T)
  delta.dm=matrix(rep(temp,nsrvys),nrow=nsrvys,ncol=ncol(delta_mat), byrow=TRUE)
  delta.dm=simplify_dm(delta.dm,temp.pao);
  temp=grep("p1.",colnames(temp.pao$survcov),fixed=TRUE,value=T)
  p.dm=matrix(rep(temp,nsrvys),nrow=nsrvys,ncol=ncol(p_mat),byrow=TRUE)
  temp=grep("p2.",colnames(temp.pao$survcov),fixed=TRUE,value=T)
  p.dm=rbind(p.dm,matrix(rep(temp,nsrvys),nrow=nsrvys,ncol=ncol(p_mat),byrow=T))
  p.dm=simplify_dm(p.dm,temp.pao)
  v=paste0("p1(",unlist(lapply(1:ns, function(xx){rep(xx,nsrvyseasn[xx])})),
           "-",  unlist(lapply(1:ns, function(xx){1:nsrvyseasn[xx]})),")")

  if (ns>1) {
    p.dm=diagbind(delta.dm,p.dm)
    rownames(p.dm)=c(gsub('p1','delta',v),v,gsub('p1','p2',v))
    colnames(p.dm)=paste0("e",1:ncol(p.dm))

    if(!is.null(fixed)) {
      fixed$idx=match(fixed$param,unlist(lapply(list(psi.dm,NULL,NULL,NULL,p.dm),rownames)))
      i=which(fixed$idx<=nrow(psi.dm))
      if (length(i)>0) {
        psi.dm[fixed$idx[i],]=0
      }
    }
    rv=runPresence(temp.pao,list(psi.dm,NULL,NULL,NULL,p.dm,NULL),
                              model=model,modname=modname,
                              fixed=fixed,initvals=initvals,outfile=outfile,miscopts=miscopts)
  } else {
    rownames(p.dm)=c(v,gsub('p1','p2',v));   colnames(p.dm)=paste0("b",1:ncol(p_mat))
    rownames(delta.dm)=gsub('p1','delta',v); colnames(delta.dm)=paste0("c",1:ncol(delta.dm))

    psi.dm=simplify_dm(psi.dm,temp.pao);  p.dm=simplify_dm(p.dm,temp.pao); delta.dm=simplify_dm(delta.dm,temp.pao)

    if(!is.null(fixed)){
      fixed$idx=match(fixed$param,unlist(lapply(list(psi.dm,p.dm,delta.dm),rownames)))
    }

    rv=runPresence(temp.pao,list(psi.dm,p.dm,delta.dm,NULL,NULL,NULL),
                              model=model,modname=modname,
                              fixed=fixed,initvals=initvals,outfile=outfile,miscopts=miscopts)
  }

  ################################################################
  #### Extract results from output file
  ################################################################

  npar=(ncol(psi.dm)+ncol(p.dm)); if (ns==1) npar=ncol(psi.dm)+ncol(p.dm)+ncol(delta.dm)
  v=readLines(outfile); l=length(v);

  ### get PRESENCE version
  i=grep("Version",v,fixed=TRUE)[1]; version=gsub(".+Version ","",v[i])

  ### get design matrices from output file
  dmats <- get_dmats(v)
  psi.dm=dmats[[1]];
  if (length(dmats)>4) p.dm=dmats[[5]]
  else p.dm=dmats[[2]]

  ## find AIC value for each model
  i=grep("-2log.likelihood",v)
  neg2loglike=as.numeric(gsub(".+= ","",v[i]))
  aic=as.numeric(gsub(".+= ","",v[i+1]))
  v=v[-1:-i]

  ### get Untransformed coefficients (Beta's)
  i=grep("^A.+ : ",v); psi.coeff=getbetas(v[i],psi.dm)
  j=grep('[C]*R',rownames(psi.coeff)); R.coeff=psi.coeff[j,]; psi.coeff=psi.coeff[-j,];
  i=grep("^A.+[C]*psi.+ :",v); j=grep("^A.+[C]*R.+ :",v)
  k=grep("^[B-F][0-9].+ : ",v);
  if (ns<=1) p.coeff=getbetas(v[k],diagbind(p.dm,delta.dm)) else p.coeff=getbetas(v[k],p.dm)
  l=grep("^delta",rownames(p.coeff)); delta.coeff=p.coeff[l,]; p.coeff=p.coeff[-l,]
  k=grep("^[B-F].+ p.+ :",v); l=grep("^[B-F].+ delta.+ :",v)

  ### get_VC matrix
  ii=grep("^[A-F][0-9].+ : ",v); names=gsub(" .+","",v[ii]); npar=length(names)
  instns=grep("Variance-Covariance Matrix of",v,fixed=TRUE)
  VC=get_VC(v=v,offset=instns+1,n.coeff=npar,cov.names=names)

  psi.VC=as.matrix(VC[1:length(i), 1:length(i)])
  R.VC  =as.matrix(VC[length(i)+1:length(j), length(i)+1:length(j)])
  p.VC  =as.matrix(VC[length(c(i,j))+1:length(k), length(c(i,j))+1:length(k)])
  delta.VC  =as.matrix(VC[length(c(i,j,k))+1:length(l), length(c(i,j,k))+1:length(l)])

  psi.est=p.est=psi_c.est=R.est=delta.est=NULL
  VCoutopt=miscopts[1]
  if (! (VCoutopt %in% c("nose","betavc","noreal"))) {  ## return real parameters
    ## Calc real parameter estimates
    psilbl=paste0('psi_',data$unitnames)
    if (ns>1) {
      psilbl=c('psi',paste0(rep(paste0('Cpsi',0:2),ns1),'(',rep(1:ns1,each=3),')')); l=length(psilbl)
      psilbl=apply(cbind(rep(psilbl,each=nunits),rep(data$unitnames,l)),1,paste,collapse='_')
    }
    x=psi_mat; if (ns>1) x=diagbind(psi_mat,Cpsi_mat)
    psi.est=calc_real(cov=x,coeff=psi.coeff$est,VC=psi.VC,conf=conf,rownames=psilbl)
    x=R_mat; if (ns>1) x=diagbind(R_mat,CR_mat)
    R.est  =calc_real(cov=x,coeff=R.coeff$est,VC=R.VC,conf=conf,rownames=gsub('psi','R',psilbl))
    plbl=c(paste0('p1(',1:nsrvys,')'),paste0('p2(',1:nsrvys,')')); l=length(plbl)
    plbl=apply(cbind(rep(plbl,each=nunits),rep(data$unitnames,l)),1,paste,collapse='_')
    p.est=calc_real(cov=p_mat,coeff=p.coeff$est,VC=p.VC,conf=conf,rownames=plbl,fixed=fixed)
    dlbl=gsub('p1','dlta',plbl[grep('p1',plbl)])
    delta.est=calc_real(cov=delta_mat,coeff=delta.coeff$est,VC=delta.VC,conf=conf,rownames=dlbl,fixed=fixed)
  }
  ##### check for warnings
  warn.conv=check_conv_warn(v,quiet=length(grep('quiet',miscopts)))
  warn.VC=check_VC_warn(v)

  result=list(modname=modname,
               model=list(psi=psi,R=R,Cpsi=Cpsi,CR=CR,delta=delta,p=p),dmat=list(psi=psi.dm,p=p.dm),
               data=temp.pao,outfile=outfile,
               neg2loglike=neg2loglike,npar=npar, aic=aic,
               beta=list(psi=  as.data.frame(psi.coeff),  psi.VC=psi.VC,
                         R  =  as.data.frame(R.coeff),    R.VC=  R.VC,
                         delta=as.data.frame(delta.coeff),delta.VC=delta.VC,
                         p  =  as.data.frame(p.coeff),    p.VC=p.VC,VC=VC),
               real=list(psi=as.data.frame(psi.est),
                         R=as.data.frame(R.est),
                         delta=as.data.frame(delta.est),
                         p=as.data.frame(p.est),
               warnings=list(conv=warn.conv,VC=warn.VC)))
  if (ns<=1) result$dmat$delta=delta.dm

  class(result)=c("occMod","doMs2")
  return(result)

}
