occMod_RPC<-function(lambda=call(),lambda.cov=data$unitcov,
                     r=call(),r.cov=cbind(data$unitcov,data$survcov), nic=FALSE,
                     modname=NULL,outfile,model=100,fixed=NULL,initvals=NULL,data,conf,miscopts){
  #' Fit static occupancy/single season model
  #'
  #' This is not intended for direct use, but instead the \code{\link{occMod}} function should be used with \code{type="so"}.
  #'
  #'@param lambda the right-hand side of the formula for the model to fit for average individuals/site
  #'@param lambda.cov a data frame containing the unit-specific covariates to use for lambda
  #'component of the model.
  #'@param r the right-hand side of the formula for the model to fit for detection probability.
  #'@param r.cov a data frame containing the survey-specific covariates to use for the detection
  #'component of the model.
  #'@param modname (optional) a string containing the model name
  #'@param outfile name for output file (use outfile='modname') for outfile named via model name
  #'@param model the PRESENCE model code. DO NOT CHANGE.
  #'@param fixed a single-column matrix containing values for real parameters to be fixed at.
  #'\code{rownnames(fixed)} should contain the index of the real parameters to be fixed.
  #'@param initvals initial values for the beta parameters at which PRESENCE begins the optimisation.
  #'The default values in PRESENCE is 0.
  #'@param data the \code{pao} data object containing the detection data and other information.
  #'@param conf level for confidence interval (may be vector valued).
  #'@param miscopts miscelleneous options (eg., maxfn=32000)
  #'@param nic option to select Royle-Nichols heterogenity model instead of Royle point-count model
  #' @details Pre-defined covariates:
  #' \itemize{
  #' \item{SURVEY} {categorical covariate indicating survey (for detection parameters)}
  #' }
  #'
  #'@return returns a list of class \code{"occMod"} and \code{"rpc"}.
  #'
  #'\code{occMod$beta} contains the objects:
  #'  #'\item{lambda}{estimated logistic regression coefficients and standard errors for avg. individuals/site.}
  #'\item{lambda.VC}{variance-covariance matrix for logistic regression coefficients for lambda.}
  #'\item{r}{estimated logistic regression coefficients and standard errors for probability of detection.}
  #'\item{r.VC}{variance-covariance matrix for logistic regression coefficients for probability of detection.}
  #'\item{VC}{the full variance-covariance matrix for all logistic regression coefficients.}
  #'
  #'\code{occMod$real} contains the objects:
  #'\item{lambda}{estimated avg. individuals/site for each sampling unit, along with standard
  #'errors and limits of 95\% confidence interval.}
  #'\item{r}{estimated individual probabilities of detection for each survey, along with standard errors and
  #'limits of 95\% confidence interval.}
  #'
  #'\code{occMod$derived} contains the objects:
  #'\item{lambda_c}{estimated probabilities of occurrence given the detection history for each sampling
  #'unit, along with standard errors and limits of 95\% confidence interval. Will be \code{=1} for
  #'any unit where the species was detected at least once.}
  #'
  #'@author Darryl MacKenzie and Jim Hines
  #'
  #' @examples
  #'
  #' N=500      # sites
  #' K=6        # surveys
  #' lambda=2  # number of individuals per site
  #' p=.2       # prob(detect inidividual per site,survey)
  #'
  #' pop=rpois(N,lambda) # simulate number of individuals per site
  #'
  #' count=NULL          # for each survey ...
  #' for (k in 1:K) {    #    simulate number of individuals detected
  #'   count=cbind(count, rbinom(N, pop, p))
  #' }
  #'
  #' pao=createPao(count)
  #'
  #' m2=occMod(list(lambda~1, r~1), data=pao, type='rpc', outfile='modname')
  #' print(print_one_site_estimates(m2))
  #' print(print_one_site_estimates(m2,derived=T))
  #'
  #'@seealso \code{\link{occMod}}, \code{pao}

  options("na.action"="na.pass")
  if (is.null(lambda.cov))
    lambda_mat=model.matrix(lambda,data.frame(rep(1,data$nunits)))
  else
    lambda_mat=model.matrix(lambda,lambda.cov)
  rc='r'; if (nic) rc='c'
  if (is.null(r.cov))
    p_mat=model.matrix(r,data.frame(SURVEY=as.factor(rep(1:data$nsurveys,each=data$nunits))))
  else
    p_mat<-model.matrix(r,r.cov)

  ## create and output temporary pao file
  temp.pao=data
  temp.pao$nunitcov=ncol(lambda_mat); temp.pao$unitcov=as.data.frame(lambda_mat)
  temp.pao$nsurvcov=ncol(p_mat)
  temp.pao$survcov=as.data.frame(p_mat)
  temp.pao$paoname=ifelse(is.null(data$paoname),"paodata.pao",data$paoname)

  colnames(temp.pao$unitcov)=gsub("[(]Intercept[)]","int",colnames(temp.pao$unitcov))
  colnames(temp.pao$unitcov)=paste("lambda.",colnames(temp.pao$unitcov),sep="")
  colnames(temp.pao$survcov)=gsub("[(]Intercept[)]","int",colnames(temp.pao$survcov))
  colnames(temp.pao$survcov)=paste("r.",colnames(temp.pao$survcov),sep="")

  ## create design matrices file
  lambda.dm=matrix(colnames(temp.pao$unitcov),1,temp.pao$nunitcov)
  lambda.dm.col=ncol(lambda.dm)
  rownames(lambda.dm)="lambda"
  colnames(lambda.dm)=paste("a",1:temp.pao$nunitcov,sep="")

  r.dm=matrix(rep(colnames(temp.pao$survcov),temp.pao$nsurveys),
               nrow=temp.pao$nsurveys,ncol=temp.pao$nsurvcov,byrow=TRUE);
  r.dm.col=ncol(r.dm)
  rownames(r.dm)=paste0(rc,1:temp.pao$nsurveys)
  colnames(r.dm)=paste0("b",1:ncol(r.dm))
  npar=lambda.dm.col+r.dm.col
  lambda.dm=simplify_dm(lambda.dm,temp.pao);  r.dm=simplify_dm(r.dm,temp.pao)

  if(!is.null(fixed)){
    rnames=unlist(lapply(list(lambda.dm,r.dm),rownames))
    fixed$idx<-match(fixed$param,rnames)
    if (sum(is.na(fixed$idx))>0) cat('fixed names must be in ',rnames,'\n')
  }
  rv=runPresence(temp.pao,list(lambda.dm,r.dm,NULL,NULL,NULL,NULL),
                model,modname=modname,fixed=fixed,initvals=initvals,outfile=outfile,miscopts=miscopts)

  ### extract results from output file
  v=readLines(outfile)

  warn.conv=check_conv_warn(v); warn.VC=check_VC_warn(v)   ##### check for warnings

  ### get PRESENCE version
  i=grep("Version",v,fixed=TRUE)[1]; version=gsub(".+Version ","",v[i])

  ## find AIC value for each model
  i=grep("-2log.likelihood",v); neg2loglike=as.numeric(gsub(".+= ","",v[i])); aic=as.numeric(gsub(".+= ","",v[i+1]))
  v=v[-1:-i]

  ### get Untransformed coefficients (Beta's)
  i=grep("^A.+lambda.+ : ",v); lambda.coeff=getbetas(v[i],lambda.dm)
  j=grep("^B.+ : ",v); r.coeff=getbetas(v[j],r.dm)

  ### get_VC matrix
  ii=grep("^[A-F].+ : ",v); names=gsub(" .+","",v[ii]); npar=length(names)
  instns=grep("Variance-Covariance Matrix of",v,fixed=TRUE)
  VC=get_VC(v=v,offset=instns+1,n.coeff=npar,cov.names=names)

  lambda.VC=as.matrix(VC[1:length(i), 1:length(i)])
  r.VC  =as.matrix(VC[length(i)+1:length(j), length(i)+1:length(j)])

  lambda.est=r.est=lambda_c.est=psi=N=NULL
  VCoutopt=miscopts[1]
  if (! (VCoutopt %in% c("nose","betavc","noreal"))) {  ## return real parameters
    ## Calc real parameter estimates
    lambda.est=calc_real(cov=lambda_mat,coeff=lambda.coeff$est,VC=lambda.VC,conf=conf,link='exp',rownames=paste0('lambda_',temp.pao$unitnames))
    s=paste0(rc,rep(1:temp.pao$nsurveys,each=temp.pao$nunits),'_',rep(temp.pao$unitnames,temp.pao$nsurveys))
    r.est=calc_real(cov=p_mat,coeff=r.coeff$est,VC=r.VC,conf=conf,rownames=s,fixed=fixed)

    i=grep('psi.+:',v)
    if (length(i)>0) {
      psi=matrix(as.numeric(unlist(strsplit(gsub(' +',',',gsub('.+:','',gsub('-','',v[i]))),','))[-1]),
                 ncol=4,byrow=T,
                 dimnames=list(gsub(' +',' ',gsub(':.+','',v[i])),
                               c('est','se','lower_0.95','upper_0.95')))
    }
    i=grep('N[(].+:',v)
    if (length(i)>0) {
      N=matrix(as.numeric(unlist(strsplit(gsub(' +',',',gsub('.+:','',gsub('-','',v[i]))),','))[-1]),
                ncol=4,byrow=T,dimnames=list(
                  gsub(' +',' ',gsub(':.+','',v[i])),
                  c('est','se','lower_0.95','upper_0.95')))
    }

  } # end if(!noReal)
  derived=list(psi=data.frame(psi), N=data.frame(N))
  #  get gof results if desired
  gof=list(chat=NA,TS=NA,tbl=NULL,bs=NA)
  if (length(grep('boot2=',miscopts))>0) {
    i=grep('boot2=',miscopts); modfitboot=as.numeric(gsub('.+=','',miscopts[i]))
    i=grep("  Chi-square",v); j=grep("Test Statistic =",v); n=j-i-1
    v2=gsub(' +',' ',v[i+1:n])
    v5=lapply(1:n,function(ii){
      v3=unlist(strsplit(v2[ii],')',fixed=T));
      v4=c(paste0(v3[1],')'),as.numeric(unlist(strsplit(v3[2],' '))[-1]))
    })
    Prb=as.numeric(gsub('.+= +','',v[grep("stic >= obs",v)]))
    TS=as.numeric(gsub(' min.+','',gsub('.+= +','',v[grep("tic .data",v)])))
    TS_low=as.numeric(gsub('.+= +','',v[grep("Lowest",v)]))
    TS_hi=as.numeric(gsub('.+= +','',v[grep("Highest",v)]))
    TS=c(TS,TS_low,TS_hi); names(TS)=c('TestStat','TS_low','TS_High')
    j=grep('of c-hat',v); chat=as.numeric(gsub('.+= +','',gsub(' +[(]=.+','',v[j])))
    tbl=matrix(unlist(v5),ncol=4,byrow=T); colnames(tbl)=c('History','Observed','Expected','Chi-square')
    gof=list(chat=chat,TS=TS,Prb=Prb,tbl=tbl,bootstraps=modfitboot)
  }

  result=list(modname=modname, model=list(lambda=lambda,r=r), dmat=list(lambda=lambda.dm,r=r.dm),
               data=temp.pao, outfile=outfile, neg2loglike=neg2loglike, npar=npar, aic=aic,
               beta=list(lambda=as.data.frame(lambda.coeff),lambda.VC=lambda.VC,
                         r=as.data.frame(r.coeff),r.VC=r.VC, VC=VC),
               real=list(lambda=as.data.frame(lambda.est), r=as.data.frame(r.est)),
               derived=derived, gof=gof,
               warnings=list(conv=warn.conv,VC=warn.VC),
               version=list(PRESENCE=version,RPresence=packageVersion("RPresence")))
  if (nic) {
    names(result$model)[2]=names(result$dmat)[2]=names(result$beta)[3]=names(result$real)[2]="c"
    names(result$beta)[4]="c.VC"
  }
  class(result)=c("occMod","so")
  return(result)
}
