occMod_SO_het<-function(psi=call(),psi.cov=data$unitcov,
                     p=call(),p.cov=cbind(data$unitcov,data$survcov), pi=call(),
                     modname=NULL,outfile,model=100,fixed=NULL,initvals=NULL,data,conf,miscopts){
  #' Fit static occupancy/single season model
  #'
  #' This is not intended for direct use, but instead the \code{\link{occMod}} function should be used with \code{type="so"}.
  #'
  #'@param psi the right-hand side of the formula for the model to fit for occupancy probability.
  #'@param psi.cov a data frame containing the unit-specific covariates to use for the occupancy
  #'component of the model.
  #'@param p the right-hand side of the formula for the model to fit for detection probability.
  #'@param p.cov a data frame containing the survey-specific covariates to use for the detection
  #'component of the model.
  #'@param pi hetergeneity pi parameter (prop sites in group 1).
  #'@param modname (optional) a string containing the model name
  #'@param outfile name for output file (use outfile='modname') for outfile named via model name
  #'@param model the PRESENCE model code. DO NOT CHANGE.
  #'@param fixed a single-column matrix containing values for real parameters to be fixed at.
  #'\code{rownnames(fixed)} should contain the index of the real parameters to be fixed.
  #'@param initvals initial values for the beta parameters at which PRESENCE begins the optimisation.
  #'The default values in PRESENCE is 0.
  #'@param data the \code{pao} data object containing the detection data and other information.
  #'@param conf level for confidence interval (may be vector valued).
  #'@param miscopts char vector of options to send to Presence (eg., maxfn=32000)- last element must be ''.  #'
  #'@return returns a list of class \code{"occMod"} and \code{"so"}.
  #'
  #'\code{occMod$beta} contains the objects:
  #'  #'\item{psi}{estimated logistic regression coefficients and standard errors for probability of occurrence.}
  #'\item{psi.VC}{variance-covariance matrix for logistic regression coefficients for probability of occurrence.}
  #'\item{p}{estimated logistic regression coefficients and standard errors for probability of detection.}
  #'\item{p.VC}{variance-covariance matrix for logistic regression coefficients for probability of detection.}
  #'\item{VC}{the full variance-covariance matrix for all logistic regression coefficients.}
  #'
  #'\code{occMod$real} contains the objects:
  #'\item{psi}{estimated probabilities of occurrence for each sampling unit, along with standard
  #'errors and limits of 95\% confidence interval.}
  #'\item{p}{estimated probabilities of detection for each survey, along with standard errors and
  #'limits of 95\% confidence interval.}
  #'
  #'\code{occMod$derived} contains the objects:
  #'\item{psi_c}{estimated probabilities of occurrence given the detection history for each sampling
  #'unit, along with standard errors and limits of 95\% confidence interval. Will be \code{=1} for
  #'any unit where the species was detected at least once.}
  #'
  #'@author Darryl MacKenzie
  #'
  #'@seealso \code{\link{occMod}}, \code{pao}
  #' @examples
  #'
  #' # load a csv data file
  #' filename<-system.file("extdata/bobcat.csv",package="RPresence")
  #' csv<-read.csv(filename,as.is=T,header=F)
  #' ##
  #' nsites=nrow(csv); nsrvys=ncol(csv)  #  set number of sites,surveys from det. data
  #' dethist=matrix(as.integer(unlist(csv)),nrow=nsites)
  #' ##          create input "pao" object, for use with occMod function
  #' data=createPao(dethist, title="single-sp, detection heterogeneity example")
  #'
  #' mod1<-occMod(model=list(psi~1,p~1,pi~1),data=data,type="so.het")
  #'

  options("na.action"="na.pass")
  n=data$nunits; k=data$nsurveys
  if (is.null(psi.cov))
    psi_mat=model.matrix(psi,data.frame(rep(1,n))) else
    psi_mat=model.matrix(psi,psi.cov)

  if (is.null(p.cov))
    p.cov=data.frame(SURVEY=as.factor(rep(1:k,each=n)))
  else{
    p.cov$SURVEY=as.factor(rep(1:k,each=n))
  }
  p.cov$GRP="A"
  p.cov=rbind(p.cov,p.cov)
  p.cov$GRP[n*k+1:(n*k)]="B"
  p_mat<-model.matrix(p,p.cov)

  pi_mat=model.matrix(pi,data.frame(rep(1,n)))
  ## create and output temporary pao file
  temp.pao=data
  temp.pao$nunitcov=ncol(psi_mat); temp.pao$unitcov=as.data.frame(psi_mat)
  temp.pao$nsurvcov=ncol(p_mat)
  temp.pao$survcov=as.data.frame(p_mat)
  temp.pao$paoname=ifelse(is.null(data$paoname),"paodata.pao",data$paoname)

  colnames(temp.pao$unitcov)=gsub("[(]Intercept[)]","int",colnames(temp.pao$unitcov))
  colnames(temp.pao$unitcov)=paste("psi.",colnames(temp.pao$unitcov),sep="")
  colnames(temp.pao$survcov)=gsub("[(]Intercept[)]","int",colnames(temp.pao$survcov))
  colnames(temp.pao$survcov)=paste("p.",colnames(temp.pao$survcov),sep="")

  ## create design matrices file
  psi.dm=matrix(colnames(temp.pao$unitcov),1,temp.pao$nunitcov)
  psi.dm.col=ncol(psi.dm)
  rownames(psi.dm)="psi"
  colnames(psi.dm)=paste("a",1:temp.pao$nunitcov,sep="")

  p.dm=matrix(rep(colnames(temp.pao$survcov),k*2),ncol=temp.pao$nsurvcov,byrow=TRUE);
  rownames(p.dm)=c(paste0("p1",1:k),paste0("p2",1:k))
  colnames(p.dm)=paste0("b",1:ncol(p.dm))
  pi.dm=matrix(1,1,1); colnames(pi.dm)='c1'; rownames(pi.dm)='pi'
  npar=ncol(psi.dm)+ncol(p.dm)+1
  psi.dm=simplify_dm(psi.dm,temp.pao);
  p.dm=simplify_dm(p.dm,temp.pao,het=1)

  if(!is.null(fixed)){
    rnames=unlist(lapply(list(psi.dm,p.dm,pi.dm),rownames))
    fixed$idx<-match(fixed$param,rnames)
    if (sum(is.na(fixed$idx))>0) cat('fixed names must be in ',rnames,'\n')
  }
  rv=runPresence(temp.pao,list(psi.dm,p.dm,pi.dm,NULL,NULL,NULL),
                model,modname=modname,fixed=fixed,initvals=initvals,outfile=outfile,miscopts=miscopts)

  ### extract results from output file
  v=readLines(outfile)

  warn.conv=check_conv_warn(v); warn.VC=check_VC_warn(v)   ##### check for warnings

  ### get PRESENCE version
  i=grep("Version",v,fixed=TRUE)[1]; version=gsub(".+Version ","",v[i])

  ## find AIC value for each model
  i=grep("-2log.likelihood",v);
  neg2loglike=as.numeric(gsub(".+= ","",v[i]));
  aic=as.numeric(gsub(".+= ","",v[i+1]))
  v=v[-1:-i]

  ### get Untransformed coefficients (Beta's)
  i=grep("^A.+psi.+ : ",v); psi.coeff=getbetas(v[i],psi.dm)
  j=grep("^B.+ : ",v); p.coeff=getbetas(v[j],p.dm)
  m=grep("^C.+ : ",v); pi.coeff=getbetas(v[m],pi.dm)

  ### get_VC matrix
  ii=grep("^[A-F].+ : ",v); names=gsub(" .+","",v[ii]); npar=length(names)
  instns=grep("Variance-Covariance Matrix of",v,fixed=TRUE)
  VC=get_VC(v=v,offset=instns+1,n.coeff=npar,cov.names=names)

  psi.VC=as.matrix(VC[1:length(i), 1:length(i)])
  p.VC  =as.matrix(VC[length(i)+1:length(j), length(i)+1:length(j)])
  pi.VC =as.matrix(VC[nrow(VC),nrow(VC)],1,1)

  psi.est=p.est=psi_c.est=pi.est=NULL
  VCoutopt=miscopts[1]
  if (! (VCoutopt %in% c("nose","betavc","noreal"))) {  ## return real parameters
    ## Calc real parameter estimates
    psi.est=calc_real(cov=psi_mat,coeff=psi.coeff$est,VC=psi.VC,conf=conf,rownames=paste0('psi_',temp.pao$unitnames))
    s=paste0('p1(',rep(1:k,each=n),')',rep(temp.pao$unitnames,k))
    s=c(s,gsub('p1','p2',s))
    p.est=calc_real(cov=p_mat,coeff=p.coeff$est,VC=p.VC,conf=conf,rownames=s,fixed=fixed)

    pi.est=calc_real(cov=pi_mat,coeff=pi.coeff$est,VC=pi.VC,conf=conf,
                     rownames=paste0('pi_',temp.pao$unitnames),fixed=fixed)

    nk=n*k; ii=which(rowSums(temp.pao$det.data,na.rm=TRUE)>0)
    calc_psi_c <- function(b) {
      psiest = plogis(psi_mat %*% b[1:length(psi.coeff$est)])
      pest=plogis(p_mat %*% b[length(psi.coeff$est)+1:length(p.coeff$est)])
      piest=plogis(pi_mat %*% b[length(b)])
      qstar1=apply(1-matrix(pest[1:nk],ncol=k),1,prod)
      qstar2=apply(1-matrix(pest[nk+1:nk],ncol=k),1,prod)
      qstar=piest*qstar1+(1-piest)*qstar2
      rv=psiest*qstar/(psiest*qstar+1-psiest)
      rv[ii]=1
      return(rv)
    }
    b=c(psi.coeff$est,p.coeff$est,pi.coeff$est)
    psi_c.est=calc_psi_c(b); grad=matrix(0,length(psi_c.est), length(b))
    for (i in 1:length(b)) {
      b[i]=b[i]+.1e-9
      psi_c_tmp = calc_psi_c(b)
      grad[,i] = (psi_c_tmp - psi_c.est) /.1e-9
      b[i]=b[i] - .1e-9
    }
    newvc = grad %*% VC %*% t(grad)
    se=sqrt(diag(newvc)); se=se/psi_c.est/(1-psi_c.est)
    ci1=plogis(qlogis(psi_c.est)-1.96*se); ci2=plogis(qlogis(psi_c.est)+1.96*se)
    psi_c = cbind(psi_c.est, se, ci1, ci2); rownames(psi_c)=temp.pao$unitnames
  } # end if(!noReal)
  #  get gof results if desired
  gof=list(chat=NA,TS=NA,tbl=NULL,bs=NA)
  if (length(grep('boot2=',miscopts))>0) {
    i=grep('boot2=',miscopts); modfitboot=as.numeric(gsub('.+=','',miscopts[i]))
    i=grep("  Chi-square",v); j=grep("Test Statistic =",v); n=j-i-1
    v2=gsub(' +',' ',v[i+1:n])
    v5=lapply(1:n,function(ii){
      v3=unlist(strsplit(v2[ii],')',fixed=T));
      v4=c(paste0(v3[1],')'),as.numeric(unlist(strsplit(v3[2],' '))[-1]))
    })
    Prb=as.numeric(gsub('.+= +','',v[grep("stic >= obs",v)]))
    TS=as.numeric(gsub(' min.+','',gsub('.+= +','',v[grep("tic .data",v)])))
    TS_low=as.numeric(gsub('.+= +','',v[grep("Lowest",v)]))
    TS_hi=as.numeric(gsub('.+= +','',v[grep("Highest",v)]))
    TS=c(TS,TS_low,TS_hi); names(TS)=c('TestStat','TS_low','TS_High')
    j=grep('of c-hat',v); chat=as.numeric(gsub('.+= +','',gsub(' +[(]=.+','',v[j])))
    tbl=matrix(unlist(v5),ncol=4,byrow=T); colnames(tbl)=c('History','Observed','Expected','Chi-square')
    gof=list(chat=chat,TS=TS,Prb=Prb,tbl=tbl,bootstraps=modfitboot)
  }


  result=list(modname=modname, model=list(psi=psi,p=p), dmat=list(psi=psi.dm,p=p.dm),
               data=temp.pao, outfile=outfile, neg2loglike=neg2loglike, npar=npar, aic=aic,
               beta=list(psi=as.data.frame(psi.coeff),psi.VC=psi.VC,
                         p=as.data.frame(p.coeff),p.VC=p.VC, VC=VC),
               real=list(psi=as.data.frame(psi.est), p=as.data.frame(p.est),
                         pi=data.frame(pi.est)),
               derived=list(psi_c=psi_c), gof=gof,
               warnings=list(conv=warn.conv,VC=warn.VC),
               version=list(PRESENCE=version,RPresence=packageVersion("RPresence")))
  class(result)=c("occMod","so")
  return(result)
}
