% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/occ.mod.do.ms2.R
\name{occMod_DO_ms2}
\alias{occMod_DO_ms2}
\title{Fit multi-state dynamic occupancy (multi-season) occupancy model using the second (conditional binomial) parameterisation in PRESENCE.}
\usage{
occMod_DO_ms2(
  psi = call(),
  psi.cov = data$unitcov,
  Cpsi = call(),
  Cpsi.cov = NULL,
  R = call(),
  R.cov = data$unitcov,
  CR = call(),
  CR.cov = NULL,
  p = call(),
  p.cov = data$survcov,
  delta = call(),
  delta.cov = data$survcov,
  modname = NULL,
  outfile,
  model = 6100,
  fixed = NULL,
  initvals = NULL,
  data,
  conf,
  miscopts
)
}
\arguments{
\item{psi}{the right-hand side of the formula for the model to fit for initial occupancy probability.}

\item{psi.cov}{a data frame containing the unit-specific covariates to use for the occupancy component of the model.}

\item{Cpsi}{the right-hand side of the formula for the model to fit for occupancy probability, both first year and dynamic occupancy.
The terms \code{SEASON}, \code{DYN} and \code{PREV_STATE} can each be used without having been defined in \code{psi.cov}.
\code{SEASON} allows for a seasonal effect, \code{DYN} enables dynamic occupancy probabilities to be different from first year occupancy,
and \code{PREV_STATE} allows the probability to be different depending on the state in the previous season.}

\item{Cpsi.cov}{a data frame containing the unit-specific covariates to use for the occupancy component of the model.}

\item{R}{the right-hand side of the formula for the model to fit for the probability of being in the second state, conditional on the unit being occupied.}

\item{R.cov}{a data frame containing the unit-specific covariates to use for the probability of being in the second state, conditional on the unit being occupied part of the model.}

\item{CR}{the right-hand side of the formula for the model to fit for the probability of being in the second state, conditional on the unit being occupied.
The same built-in covariates, \code{SEASON}, \code{DYN} and \code{PREV_STATE} can be used for CR.}

\item{CR.cov}{a data frame containing the unit-specific covariates to use for the probability of being in the second state, conditional on the unit being occupied part of the model.}

\item{p}{the right-hand side of the formula for the model to fit for detection probability.
The terms \code{SEASON} and \code{STATE} can each be used without having been defined in \code{p.cov}.
\code{SEASON} allows for a seasonal effect, and \code{STATE} allows the probability of detection to be different depending on the state in the current season.}

\item{p.cov}{a data frame containing the survey-specific covariates to use for the detection component of the model.}

\item{delta}{the right-hand side of the formula for the model to fit for the probability of detecting the second state in a survey,
conditional on the species being detected in the survey. The terms \code{SEASON} and \code{STATE} can each be used as above to the same effect.}

\item{delta.cov}{a data frame containing the survey-specific covariates to use for the probability of
detecting the second state in a survey, conditional on the species being detected in the survey.}

\item{modname}{(optional) a string containing the model name}

\item{outfile}{name for output file (use outfile='modname') for outfile named via model name}

\item{model}{the PRESENCE model code. DO NOT CHANGE.}

\item{fixed}{a single-column matrix containing values for real parameters to be fixed at. \code{rownnames(fixed)} should contain the index of the real parameters to be fixed.}

\item{initvals}{initial values for the beta parameters at which PRESENCE begins the optimisation. The default values in PRESENCE is 0.}

\item{data}{the \code{pao} data object containing the detection data and other information.}

\item{conf}{level for confidence interval (may be vector valued).}

\item{miscopts}{char vector of options to pass to Presence... last element must be null string.}
}
\value{
returns a list of class \code{"occMod"} and \code{"do.1"}.

\code{occMod$beta} contains the objects:
 \item{psi}{estimated logistic regression coefficients and standard errors for probability of occurrence.}
 \item{psi.VC}{variance-covariance matrix for psi.}
 \item{R}{estimated logistic regression coefficients and standard errors for probability of breeding (state=2), givven occupancy.}
 \item{R.VC}{variance-covariance matrix for R..}
 \item{p}{estimated logistic regression coefficients and standard errors for probability of detection.}
 \item{p.VC}{variance-covariance matrix for p.}
 \item{VC}{the full variance-covariance matrix for all logistic regression coefficients.}

 \code{occMod$real} contains the objects:
 \item{psi}{estimated probabilities of occurrence for each sampling unit, along with standard errors and limits of 95\% confidence interval.}
 \item{p}{estimated probabilities of detection for each survey, along with standard errors and limits of 95\% confidence interval.}
}
\description{
This is not intended for direct use, but instead the \code{\link{occMod}} function should be used with \code{type="do.ms.2"}.
}
\examples{

cat('# --- SINGLE-SEASON MULTI-STATE EXAMPLE\n')
cat('3 OCCUPANCY STATES, PSI-R PARAMETEIZATION) (PG 231 IN BOOK) ----\n')

 #    read detection history data from csv file...
 dethist<-read.csv(system.file(
 "extdata/cal_owl_multistate_data.csv",package="RPresence"),na.strings=".",
                   colClasses="numeric")[,-1]
nsites=nrow(dethist) #  set number of sites,surveys from det. history data
nsrvys=ncol(dethist)
frq=rep(1,nsites)
#  create survey covariate to categorize surveys 1-2 as 1 period
#                                    and surveys 3-5 as another period
 #    Since it's a survey covariate, it will be a NxT matrix (N=nsites, T=nsurveys)
#    The covariate matrix would be:
#         1 2 3 4 5
#  site1 [1 1 2 2 2]
#  site2 [1 1 2 2 2]
#   :    [: : : : :]
#  siteN [1 1 2 2 2]
#    Filling in by cols means we repeat "1" N times, then "1" N times,
#                  then "2" N times, then "2" N times, then "2" N times.
#    We save it as a data frame (without the matrix dimensions).
cov2=data.frame(PER=as.factor(c(rep(1,2*nsites),rep(2,3*nsites))))

#          create input "pao" object, for use with occMod function
data=createPao(data=dethist,frq=frq,survcov=cov2,title="Cal Owl example")

xmods<-occMod(model=list(psi~1,R~1,p~STATE, delta~PER),data=data,type="do.ms.2",outfile='modname')

x=rbind(unique(xmods$real$psi[,1:2]),unique(xmods$real$R[,1:2]),
         unique(xmods$real$p[,1:2]),unique(xmods$real$delta[,1:2]))
x$est=round(x$est,2); rownames(x)=c('psi','R','p1','p2','dlta1','dlta2')
xmods_chk=round(x,3)
print(xmods_chk)

cat('# --- MULTI-SEASON MULTI-STATE EXAMPLE (3 OCCUPANCY STATES, PSI-R PARAMETEIZATION) ----\n')

 hstfrq=genpresEVms(N=100,K=4,psi=.75,R=.8, Cpsi=c(.66,.1,.15),CR=c(.72,.77,0),
                  p1=.4,p2=.3,dlta=.6,nstates=3,SPS=2)
pao=createPao(data=hstfrq$hst,frq=hstfrq$frq,title="exp. value data",nsurveyseason=c(2,2))

m1=occMod(list(psi~1,Cpsi~STATE,R~1,CR~STATE,p~STATE,delta~1),
         data=pao,type='do.ms.2',outfile='modname')

cat('Occupancy estimates:\n'); print(unique(m1$real$psi))
cat('Occupancy-state2 estimates:\n'); print(unique(m1$real$R))
cat('Detection estimates:\n'); print(unique(m1$real$p))
cat('Detection-state2 estimates:\n'); print(unique(m1$real$delta))

}
\seealso{
\code{\link{occMod}}
}
\author{
Darryl MacKenzie, Jim Hines
}
