#test_mm.r
library(RPresence); #setwd('c:/tmp')

# --- SINGLE-SEASON MULTI-METHOD MODEL USING EXPECTED VALUE DATA ----\n')

genpresEVmm <- function(N=100, K=4, psi=.75, p=c(.4,.6), theta=0.5,
                        gamma=0, epsilon=0, methods=2, SPS=NA) {
  #' generate expected value detection histories for multi-method model
  #'
  #' \code{genpresEVcd} generates expected value detection history frequencies for
  #' single or multi-season correlated detection models
  #'@param N number of sites
  #'@param K total number of detection events = number of seasons * methods * SPS
  #'@param psi probability of occupancy
  #'@param p probability of detection for each method
  #'@param theta probability of local occupancy
  #'@param gamma probability of colonization
  #'@param epsilon probability of extinction
  #'@param methods number of methods per survey
  #'@param SPS number of surveys per season
  #'@export
  #'@return list(hst=det. history matrix, frq=vector of frequencies)
  #'@note This function generates at least 2^K detection-histories, so if K>8, it may take a
  #'       very long time to run.
  #'
  #'@seealso \code{\link{genpresEVms}},\code{\link{genpresSD}},
  #'          \code{\link{genpresSDms}}, \code{\link{genpresEV}}
  #'@examples
  #'    # generate expected value data for standard single-season model (no corr.det.)
  #' hstfrq=genpresEVmm(N=1000, K=4, psi=.8, p=c(.3,.3), theta=1)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  #'    # generate expected value data for single-season multi-method model
  #' hstfrq=genpresEVmm(N=1000, K=4, psi=.8, p=c(.3,.5), theta=.5, methods=2)
  #' EVpao=createPao(hstfrq$hst,frq=hstfrq$frq)                  #  create pao file
  #'
  UNOCC=1; OCC=2
  if (is.na(SPS)) SPS=K/methods
  gam_eps=c(gamma, 1-epsilon);
  last_method_in_survey=seq(methods,K,methods);
  last_survey_in_season=seq(methods*SPS,K,methods*SPS)
  e=environment();
  pp=rep(p,K)
  theta=rep(theta,K)

  srv <- function(j,n,hst,state) {
    nocc=n*(state-1);
    if (j %in% last_survey_in_season) nocc=n*gam_eps[state]
    if (j==0) nocc=n*psi
    nunocc=n-nocc; nlococc=nocc*theta[as.integer(j/methods)+1]; nlocunocc=nocc-nlococc
    cap(j+1 , nlococc   , hst , OCC   , OCC)
    cap(j+1 , nlocunocc , hst , UNOCC , OCC)
    cap(j+1 , nunocc    , hst , UNOCC , UNOCC)
  }
  cap <- function(j,n,hst,localstate, state) {
    if (n>0) {
      nlococc=n*(localstate-1)*(state-1); hst1=paste0(hst,'1')
      nlocunocc=n*(state-1)-nlococc     ; hst0=paste0(hst,'0')
      ndet=nlococc*pp[j]; notdet=nlococc-ndet
      nunocc= n-nlococc-nlocunocc
      if (j<K) {
        if (j %in% last_method_in_survey) {
          srv(j,nunocc   , hst0, UNOCC)
          srv(j,nlocunocc, hst0, OCC)
          srv(j,ndet     , hst1, OCC)
          srv(j,notdet   , hst0, OCC)
        }
        else {
          cap(j+1,nunocc   , hst0 , UNOCC , UNOCC)
          cap(j+1,nlocunocc, hst0 , UNOCC , OCC)
          cap(j+1,ndet     , hst1 , OCC   , OCC)
          cap(j+1,notdet   , hst0 , OCC   , OCC)
        }
      }
      else {
        if (is.na(e$frq[hst1])) e$frq[hst1]=ndet  else e$frq[hst1]=e$frq[hst1]+ndet
        if (is.na(e$frq[hst0]))
          e$frq[hst0]=notdet+nlocunocc+nunocc
        else
          e$frq[hst0]=e$frq[hst0]+notdet+nlocunocc+nunocc
      }
    }
    return()
  }
  e$frq=NULL; e$frq['asdf']=0
  srv(0, N, '', UNOCC)
  e$frq=e$frq[names(e$frq)!='asdf'];
  hst=matrix(as.numeric(unlist(strsplit(names(e$frq),''))),ncol=K,byrow=T)
  names(e$frq)=NULL
  return(list(hst=hst, frq=e$frq))
}

# --- SINGLE-SEASON MULTI-METHOD MODEL WITH EXPECTED VALUE DATA ----\n')

a=genpresEVmm(K=6, psi=.75, p=c(.5,.6,.52,.62,.54,.64), theta=c(.5,.55), methods=3)
a=createPao(data=a$hst, frq=a$frq, methods=3)
a$surveynames=c(paste0("1-",a$surveynames),paste0("2-",a$surveynames))
ssmm1<-occMod(data=a, type='so.mm', model=list(psi~1,theta~SURVEY,p~-1+SURVEY*METHOD))

# --- SINGLE-SEASON MULTI-METHOD MODEL WITH STOCHASTIC DATA ----\n')

a=genpresSDmm(N=5000, K=6, psi=.75, p=c(.5,.6,.52,.62,.54,.64), theta=c(.5,.55), methods=3)
a=createPao(data=a$hst, frq=a$frq, methods=3)
a$surveynames=c(paste0("1-",a$surveynames),paste0("2-",a$surveynames))
ssmm2<-occMod(data=a, type='so.mm', model=list(psi~1,theta~SURVEY,p~-1+SURVEY*METHOD))

# --- MULTI-SEASON MULTI-METHOD MODEL WITH EXPECTED VALUE DATA ----\n')

v=genpresEVmm(K=8, p=c(.5,.6), gamma=.1, epsilon=.2, SPS=2)
d=createPao(data=v$hst, frq=v$frq, methods=2, nsurveyseason=rep(2,2))
msmm1<-occMod(data=d, type='do.mm', model=list(psi~1,theta~1,epsilon~1,gamma~1,p~-1+DEVICE), quiet=TRUE)
w=round(rbind(msmm1$real$psi,msmm1$real$p,msmm1$real$theta,msmm1$real$gamma,msmm1$real$epsilon),4)
x=print_one_site_estimates(msmm1)

# --- SINGLE-SEASON MULTI-METHOD MODEL USING EXPECTED VALUE DATA ----
expect_equal(round(ssmm1$real$psi$est[1],3),0.75)
expect_equal(round(ssmm1$real$p$est[1],3),0.5)
expect_equal(round(ssmm1$real$theta$est[1],3),0.5)

# --- MULTI-SEASON MULTI-METHOD MODEL WITH EXPECTED VALUE DATA ----
expect_equal(round(msmm1$real$psi$est[1],3),0.75)
expect_equal(round(msmm1$real$theta$est[1],3),0.5)
expect_equal(round(msmm1$real$p$est[1],3),0.5)
